// Arrow 1 demo
// Two arrows, one with and one without gravity

// uses framework Cocoa
// uses framework OpenGL
#define MAIN
#include <stdlib.h>
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils4.h"
#include "glugg.h"
#include "gluggShapes.h"
#include "SimpleGUI.h"

GLuint program, shadowprogram; // Shader
mat4 projectionMatrix;


gluggModel arrow, floorModel, wallModel;

vec3 p1, p2, p3;
vec3 v1, v2, v3;

float g = 9.81;

void display(void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    mat4 cam = lookAt(0, 7, 14, 0, 4, 0, 0,1,0);
	
	// Add velocity first adds energy
	// Add velocity last removes energy, fix by move up on collision

	v1 = v1 + SetVec3(0, -g / 1000.0, 0); // Gravity only, scaled by 1000 as pseudo-time-step
//	v2 = v2 + SetVec3(0, -g / 1000.0, 0); // Skipped - no gravity

	if (p1.x < 4.0)
		p1 = p1 + v1;
	if (p2.x < 4.0)
		p2 = p2 + v2;
	if (p1.y < 0)
		p1.y = 0;

	glUseProgram(program);
	mat4 m = cam * T(p1.x, p1.y, p1.z);
    glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, m.m);
	gluggDrawModel(arrow, program);

	m = cam * T(p2.x, p2.y, p2.z);
    glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, m.m);
	gluggDrawModel(arrow, program);

	glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, cam.m);
	gluggDrawModel(floorModel, program);

	glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, cam.m);
	gluggDrawModel(wallModel, program);

// Shadows for both arrows
// with "cheapness projection", move z by y
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glUseProgram(shadowprogram);

	m = cam * T(p1.x,0.05,p1.z) * S(1,0,1);
    glUniformMatrix4fv(glGetUniformLocation(shadowprogram, "modelviewMatrix"), 1, GL_TRUE, m.m);
	gluggDrawModel(arrow, shadowprogram);

	m = cam * T(p2.x,0.05,p2.z) * S(1,0,1);
    glUniformMatrix4fv(glGetUniformLocation(shadowprogram, "modelviewMatrix"), 1, GL_TRUE, m.m);
	gluggDrawModel(arrow, shadowprogram);

	glDisable(GL_BLEND);

	
	sgDraw();

    glutSwapBuffers();
}

float startVelocity = 0.3;

void Reset()
{
	p1 = SetVec3(-5, 5, 2);
	p2 = SetVec3(-5 ,5, -2);
	v1 = SetVec3(startVelocity,0.0,0);
	v2 = SetVec3(startVelocity,0.0,0);
}

void myinit()
{
    glEnable(GL_DEPTH_TEST); /* enable z buffer */
    glDisable(GL_CULL_FACE);

	program = loadShaders("sphere.vert", "sphere.frag");
	shadowprogram = loadShaders("sphere.vert", "shadow.frag");

	gluggBegin(0);
		gluggColor(1, 1, 1);
		gluggRotate(-M_PI/2, 0, 0, 1);
		gluggCylinder(20, 1, 0.1);
		gluggTranslate(0,1,0);
		gluggCone(20, 0.2, 0.2);
	arrow = gluggBuildModel(1);

	gluggBegin(GLUGG_QUADS);
	gluggNormal(0,1,0);
	gluggColor(0.5, 1, 0.6);
	gluggVertex(5, 0, 5);
	gluggVertex(5, 0, -5);
	gluggVertex(-5, 0, -5);
	gluggVertex(-5, 0, 5);
	floorModel = gluggBuildModel(1);
	
	gluggBegin(GLUGG_QUADS);
	gluggNormal(0,1,0);
	gluggColor(0.8, 0.6, 0.4);
	gluggVertex(5, 0, 5);
	gluggVertex(5, 0, -5);
	gluggVertex(5, 5, -5);
	gluggVertex(5, 5, 5);
	wallModel = gluggBuildModel(1);
	
	
	projectionMatrix = frustum(-0.5, 0.5, -0.5, 0.5, 1.0, 60.0);
	
	glUseProgram(program);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	glUseProgram(shadowprogram);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);

    glClearColor (0.5, 0.8, 1.0, 1.0);
    
    sgSetPosition(30, 30);
    sgSetTextColor(0,0,0);
    sgSetBackgroundColor(0.5, 0.5, 0.5, 0.5);
    sgSetFrameColor(0,0,0);
    sgCreateStaticString(20, 20, "Only gravity, or without");
    sgCreateDisplayFloat(-1, -1, "Starting velocity: ", &startVelocity);
    sgCreateSlider(-1, -1, 200, &startVelocity, 0.1, 0.5);
    sgCreateButton(-1, -1, "Reset", Reset);

	Reset();
}

void mouse(int button, int state, int x, int y)
{
	sgMouse(state, x, y);
}

void mousedrag(int x, int y)
{
	sgMouseDrag(x, y);
}


int main(int argc, char **argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitContextVersion(3, 2);
    glutInitWindowSize(500, 500);
    glutRepeatingTimer(60);
    glutCreateWindow("Arrow 1");
    myinit();
    glutDisplayFunc(display);
    glutMouseFunc(mouse);
    glutMotionFunc(mousedrag);
    glutMainLoop();
}
