// Arrow 2 demo, arrow hits apple, plastic collision

// uses framework Cocoa
// uses framework OpenGL
#define MAIN
#include <stdlib.h>
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils4.h"
#include "glugg.h"
#include "gluggShapes.h"
#include "SimpleGUI.h"

GLuint program, shadowprogram; // Shader
mat4 projectionMatrix;


gluggModel arrow, floorModel, wallModel, apple;

vec3 p1;
vec3 v1;
vec3 ballpos;
float ballMass = 10.0;
float arrowMass = 5.0;
float ballsize = 2.0;
int collided = 0;

float g = 9.81;

void display(void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    mat4 cam = lookAt(0, 7, 14, 0, 4, 0, 0,1,0);
	
	// Add velocity first adds energy
	// Add velocity last removes energy, fix by move up on collision

	v1 = v1 + SetVec3(0, -g / 1000.0, 0); // Gravity only, scaled by 1000 as pseudo-time-step

	if (p1.x < 4.0 - collided*2)
	{
		p1 = p1 + v1;
		if (collided)
			ballpos = ballpos + v1; 
	}
	if (p1.y < 0)
	{
		if (collided) ballpos.y -= p1.y;
		p1.y = 0;
	}
	if (ballpos.y < 1)
	{
		if (collided) p1.y -= (ballpos.y - 1);
		ballpos.y = 1;
	}
	
	if (Norm(p1 - ballpos) < ballsize && !collided)
	{
		collided = 1;
		// Rescale velocity by masses
		v1 = v1*(arrowMass)/(arrowMass + ballMass);
	}

	glUseProgram(program);
	mat4 m = cam * T(p1.x, p1.y, p1.z);
    glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, m.m);
	gluggDrawModel(arrow, program);

	m = cam * T(ballpos.x, ballpos.y, ballpos.z);
    glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, m.m);
	gluggDrawModel(apple, program);

	glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, cam.m);
	gluggDrawModel(floorModel, program);

	glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, cam.m);
	gluggDrawModel(wallModel, program);

// Shadows for arrows and apple
// with "cheapness projection", move z by y
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glUseProgram(shadowprogram);

	m = cam * T(p1.x,0.05,p1.z) * S(1,0,1);
    glUniformMatrix4fv(glGetUniformLocation(shadowprogram, "modelviewMatrix"), 1, GL_TRUE, m.m);
	gluggDrawModel(arrow, shadowprogram);

	m = cam * T(ballpos.x, 0.05, ballpos.z)* S(1,0,1);
    glUniformMatrix4fv(glGetUniformLocation(shadowprogram, "modelviewMatrix"), 1, GL_TRUE, m.m);
	gluggDrawModel(apple, shadowprogram);

	glDisable(GL_BLEND);

	
	sgDraw();

    glutSwapBuffers();
}

float startVelocity = 0.3;

void Reset()
{
	p1 = SetVec3(-5, 5, 2);
	v1 = SetVec3(startVelocity,0.0,0);
	ballpos = SetVec3(0, 4, 2);
	collided = 0;
}

void myinit()
{
    glEnable(GL_DEPTH_TEST); /* enable z buffer */
    glEnable(GL_CULL_FACE);

	program = loadShaders("sphere.vert", "sphere.frag");
	shadowprogram = loadShaders("sphere.vert", "shadow.frag");

	gluggBegin(0);
		gluggColor(1, 1, 1);
		gluggRotate(-M_PI/2, 0, 0, 1);
		gluggCylinder(20, 1, 0.1);
		gluggTranslate(0,1,0);
		gluggCone(20, 0.2, 0.2);
	arrow = gluggBuildModel(1);

	gluggBegin(0);
		gluggColor(1, 0, 0);
		vec3 d = SetVec3(1, 0.8, 1);
		gluggEllipsoid(20, 20, d);
	apple = gluggBuildModel(1);

	gluggBegin(GLUGG_QUADS);
	gluggNormal(0,1,0);
	gluggColor(0.5, 1, 0.6);
	gluggVertex(5, 0, 5);
	gluggVertex(5, 0, -5);
	gluggVertex(-5, 0, -5);
	gluggVertex(-5, 0, 5);
	floorModel = gluggBuildModel(1);
	
	gluggBegin(GLUGG_QUADS);
	gluggNormal(0,1,0);
	gluggColor(0.8, 0.6, 0.4);
	gluggVertex(5, 0, 5);
	gluggVertex(5, 5, 5);
	gluggVertex(5, 5, -5);
	gluggVertex(5, 0, -5);
	wallModel = gluggBuildModel(1);
	
	
	projectionMatrix = frustum(-0.5, 0.5, -0.5, 0.5, 1.0, 60.0);
	
	glUseProgram(program);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	glUseProgram(shadowprogram);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);

    glClearColor (0.5, 0.8, 1.0, 1.0);
    
    sgSetPosition(30, 30);
    sgSetTextColor(0,0,0);
    sgSetBackgroundColor(0.5, 0.5, 0.5, 0.5);
    sgSetFrameColor(0,0,0);
    sgCreateStaticString(20, 20, "Only gravity, or without");
    sgCreateDisplayFloat(-1, -1, "Starting velocity: ", &startVelocity);
    sgCreateSlider(-1, -1, 200, &startVelocity, 0.1, 0.5);
    sgCreateDisplayFloat(-1, -1, "Apple mass: ", &ballMass);
    sgCreateSlider(-1, -1, 200, &ballMass, 5, 20);
    sgCreateDisplayFloat(-1, -1, "Arrow mass: ", &arrowMass);
    sgCreateSlider(-1, -1, 200, &arrowMass, 2, 10);
    sgCreateButton(-1, -1, "Reset", Reset);

	Reset();
}

void mouse(int button, int state, int x, int y)
{
	sgMouse(state, x, y);
}

void mousedrag(int x, int y)
{
	sgMouseDrag(x, y);
}

void key(unsigned char c, int x, int y)
{
	Reset();
}

int main(int argc, char **argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitContextVersion(3, 2);
    glutInitWindowSize(500, 500);
    glutRepeatingTimer(60);
    glutCreateWindow("Arrow 2");
    myinit();
    glutDisplayFunc(display);
    glutMouseFunc(mouse);
    glutMotionFunc(mousedrag);
    glutKeyboardFunc(key);
    glutMainLoop();
}
