// This unit creates a beach ball using the GLUGG library.

#include <stdlib.h>
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils4.h"
#include "glugg.h"
#include "gluggShapes.h"
#include "SimpleGUI.h"


vec3 colors[10] =
{
	vec3(1, 0, 0),
	vec3(1, 0, 1),
	vec3(1, 1, 0),
	vec3(0, 1, 0),
	vec3(0, 1, 1),
	vec3(1, 0.5, 0),
	vec3(0, 0.5, 1),
	vec3(0.5, 1, 0),
	vec3(0, 1, 0.5),
	vec3(1, 1, 1)	
};

// Make a sphere using the ellipsoid
// using gluggScale to support radius.
gluggModel badbollSphere(int hParts, int hSubSlices, int vSlices, float radius)
{
	gluggBegin(0);
	gluggScale(radius, radius, radius);
	
	for (int h = 0; h < hParts * hSubSlices; h++)
	{	
		gluggMode(GLUGG_TRIANGLE_STRIP);
		float a1 = 2.0 * M_PI / (hParts * hSubSlices) * h;
		float a2 = 2.0 * M_PI / (hParts * hSubSlices) * (h + 1);

		vec3 c = SetVec3(0, 1, 0);
//		gluggVertexv(c); // Should be needed but I messed it up.

		gluggColorv(colors[h / hSubSlices]);
		
		for (int v = 0; v < vSlices; v++) // Should be 1?
		{
			float b = M_PI / vSlices * v;
			
			vec3 p1 = SetVec3(cos(a1)*sin(b), cos(b), sin(a1)* sin(b));
			vec3 p2 = SetVec3(cos(a2)*sin(b), cos(b), sin(a2)* sin(b));
			gluggNormalv(normalize(VectorAdd(p1, p2)));
			gluggVertexv(p1);
			gluggVertexv(p2);
		}
		
		c = SetVec3(0, -1, 0);
		gluggVertexv(c);
	}
		
	return gluggBuildModel(1);
}
