// Badboll 3?
// Collisions between beach balls?

// uses framework Cocoa
// uses framework OpenGL
#define MAIN
#include <stdlib.h>
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils4.h"
#include "glugg.h"
#include "gluggShapes.h"
#include "SimpleGUI.h"
#include "beachball.h"

GLuint program, shadowprogram; // Shader
mat4 projectionMatrix;

gluggModel badboll, floorModel;

vec3 p1, p2, p3;
vec3 v1, v2, v3;

float g = 9.81;

void drawModelAt(gluggModel mdl, GLuint shader, mat4 mtx)
{
	glUseProgram(shader);
    glUniformMatrix4fv(glGetUniformLocation(shader, "modelviewMatrix"), 1, GL_TRUE, mtx.m);
	gluggDrawModel(mdl, shader);
}

void display(void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    mat4 cam = lookAt(0, 7, 14, 0, 4, 0, 0,1,0);
	
	float dt = 0.001;
	float mass = 0.1;
	float fg = -g*mass;
	float mass3 = 0.5;
	
	float fd = 0.1*M_PI*1*v2.y*v2.y/2; // drag Fd = CdAroV2/2, A = r2*M_PI, Cd = 0.1, ro = 1
	float fg2;
	if (v2.y > 0) fd = -fd; // Must be against the movement
	fg2 = -g*mass + fd;

	float fd3 = 0.1*M_PI*1*v3.y*v3.y/2; // drag Fd = CdAroV2/2, A = r2*M_PI, Cd = 0.1, ro = 1
	float fg3;
	if (v3.y > 0) fd3 = -fd3; // Must be against the movement
	fg3 = -g*mass3 + fd3;

//	if (fg2 * fd < 0) fg2 = 0;

//	if (v2.y > 0 && v2.y+a*fd
//	= -g*mass + fd; // drag Fd = CdAroV2/2
	// Note: Drag must NOT make the force switch sign!

	float a = fg/mass;
	float a2 = fg2/mass;
	float a3 = fg3/mass3;

	v1 = v1 + SetVec3(0, a*dt, 0); // Gravity only, scaled by 1000 as pseudo-time-step
	v2 = v2 + SetVec3(0, a2*dt, 0); // Will gain energy
	v3 = v3 + SetVec3(0, a3*dt, 0);

// 1. gravity only
// 2. gravity and drag
// 3. ??

	p1 = p1 + v1;
	p2 = p2 + v2;
	p3 = p3 + v3;

	if (p1.y < 1 && v1.y < 0)
	{
		v1.y = -v1.y;
		p1.y = 1;
	}
	if (p2.y < 1 && v2.y < 0)
	{
		v2.y = -v2.y;
		p2.y = 1;
	}
	if (p3.y < 1 && v3.y < 0)
	{
		v3.y = -v3.y;
		p3.y = 1;
	}
	
	drawModelAt(badboll, program, cam * T(p1.x, p1.y, p1.z));
	drawModelAt(badboll, program, cam * T(p2.x, p2.y, p2.z));
	drawModelAt(badboll, program, cam * T(p3.x, p3.y, p3.z));
	drawModelAt(floorModel, program, cam);

// Shadows for all three
// with "cheapness projection", move z by y
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	drawModelAt(badboll, shadowprogram, cam * T(p1.x,0.1,-p1.y+1) * S(1,0,1));
	drawModelAt(badboll, shadowprogram, cam * T(p2.x,0.1,-p2.y+1) * S(1,0,1));
	drawModelAt(badboll, shadowprogram, cam * T(p3.x,0.1,-p3.y+1) * S(1,0,1));
	glDisable(GL_BLEND);
	
	sgDraw();

    glutSwapBuffers();
}

void Reset()
{
	p1 = SetVec3(-4,10,0);
	p2 = SetVec3(0,10,0);
	p3 = SetVec3(4,10,0);
	v1 = SetVec3(0,0,0);
	v2 = SetVec3(0,0,0);
	v3 = SetVec3(0,0,0);
}

void myinit()
{
    glEnable(GL_DEPTH_TEST); /* enable z buffer */
    glEnable(GL_CULL_FACE);

	program = loadShaders("sphere.vert", "sphere.frag");
	shadowprogram = loadShaders("sphere.vert", "shadow.frag");

	badboll = badbollSphere(6, 10, 20, 1);
	
	// Make floor
	gluggBegin(GLUGG_QUADS);
	gluggNormal(0,1,0);
	gluggColor(0.5, 1, 0.6);
	gluggVertex(5, 0, 5);
	gluggVertex(5, 0, -15);
	gluggVertex(-5, 0, -15);
	gluggVertex(-5, 0, 5);
	floorModel = gluggBuildModel(1);
	
	projectionMatrix = frustum(-0.5, 0.5, -0.5, 0.5, 1.0, 60.0);
	
	glUseProgram(program);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	glUseProgram(shadowprogram);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);

    glClearColor (1.0, 1.0, 1.0, 1.0);
    
    sgSetPosition(30, 30);
    sgSetTextColor(0,0,0);
    sgSetBackgroundColor(0.5, 0.5, 0.5, 0.5);
    sgSetFrameColor(0,0,0);
//    sgCreateStaticString(20, 20, "Only gravity, with drag, with wind");
    sgCreateStaticString(20, 20, "Without and with drag");
    sgCreateStaticString(-1, -1, "Left: without drag");
    sgCreateStaticString(-1, -1, "Right: more mass");
    sgCreateButton(-1, -1, "Reset", Reset);

	Reset();
}


void mouse(int button, int state, int x, int y)
{
	sgMouse(state, x, y);
}

void mousedrag(int x, int y)
{
	sgMouseDrag(x, y);
}


int main(int argc, char **argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitContextVersion(3, 2);
    glutInitWindowSize(500, 500);
    glutRepeatingTimer(15);
    glutCreateWindow("Beachball 3");
    myinit();
    glutDisplayFunc(display);
    glutMouseFunc(mouse);
    glutMotionFunc(mousedrag);
    glutMainLoop();
}
