// Badboll 2
// Movement affected by gravity and drag
// Three cases with different integration and collision response

// uses framework Cocoa
// uses framework OpenGL
#include <stdlib.h>
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils3.h"
#include "glugg.h"
#include "gluggShapes.h"
#include "SimpleGUI.h"

GLuint program, shadowprogram; // Shader
mat4 projectionMatrix;

vec3 colors[10] =
{
	{1, 0, 0},
	{1, 0, 1},
	{1, 1, 0},
	{0, 1, 0},
	{0, 1, 1},
	{1, 0.5, 0},
	{0, 0.5, 1},
	{0.5, 1, 0},
	{0, 1, 0.5},
	{1, 1, 1}	
};

// Make a sphere using the ellipsoid
gluggModel badbollSphere(int hParts, int hSubSlices, int vSlices, float radius)
{
	gluggBegin(0);
	
	for (int h = 0; h < hParts * hSubSlices; h++)
	{	
		gluggMode(GLUGG_TRIANGLE_STRIP);
		float a1 = 2.0 * M_PI / (hParts * hSubSlices) * h;
		float a2 = 2.0 * M_PI / (hParts * hSubSlices) * (h + 1);

		vec3 c = SetVec3(0, 1, 0);
//		gluggVertexv(c); // Should be needed but I messed it up.

		gluggColorv(colors[h / hSubSlices]);
		
		for (int v = 0; v < vSlices; v++) // Should be 1?
		{
			float b = M_PI / vSlices * v;
			
			vec3 p1 = SetVec3(cos(a1)*sin(b), cos(b), sin(a1)* sin(b));
			vec3 p2 = SetVec3(cos(a2)*sin(b), cos(b), sin(a2)* sin(b));
			gluggNormalv(normalize(VectorAdd(p1, p2)));
			gluggVertexv(p1);
			gluggVertexv(p2);
		}
		
		c = SetVec3(0, -1, 0);
		gluggVertexv(c);
	}
		
	return gluggBuildModel(1);
}

gluggModel badboll, floorModel, anvil;

vec3 p1, p2, p3;
vec3 v1, v2, v3;

float g = 9.81;

void display(void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    mat4 cam = lookAt(0, 7, 14, 0, 4, 0, 0,1,0);
	
	GLfloat angle = glutGet(GLUT_ELAPSED_TIME) / 1000.0;

	// Add velocity first adds energy
	// Add velocity last removes energy, fix by move up on collision

	v1 = VectorAdd(v1, SetVec3(0, -g / 1000.0, 0)); // Gravity only, scaled by 1000 as pseudo-time-step
	v3 = VectorAdd(v3, SetVec3(0, -g / 1000.0, 0));

	p1 = VectorAdd(p1, v1);
	p2 = VectorAdd(p2, v2);
	p3 = VectorAdd(p3, v3);

	v2 = VectorAdd(v2, SetVec3(0, -g / 1000.0, 0)); // Will gain energy

	if (p1.y < 1 && v1.y < 0)
	{
		v1.y = -v1.y;
		p1.y = 1;
	}
	if (p2.y < 1 && v2.y < 0)
	{
		v2.y = -v2.y;
		p2.y = 1;
	}
	if (p3.y < 1 && v3.y < 0)
	{
		v3.y = -v3.y; // Not moving up will lose energy
		//v3.y = 1;
	}
	
	glUseProgram(program);
	mat4 m = Mult(cam, T(p1.x, p1.y, p1.z));
    glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, m.m);
	gluggDrawModel(badboll, program);
	
	glUseProgram(program);
	m = Mult(cam, T(p2.x, p2.y, p2.z));
    glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, m.m);
	gluggDrawModel(badboll, program);

	m = Mult(cam, T(p3.x, p3.y, p3.z));
    glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, m.m);
	gluggDrawModel(anvil, program);

	glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, cam.m);
	gluggDrawModel(floorModel, program);

// Shadows for all three
// with "cheapness projection", move z by y
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glUseProgram(shadowprogram);

	m = Mult(cam, Mult(T(p1.x,0.1,-p1.y+1), S(1,0,1)));
    glUniformMatrix4fv(glGetUniformLocation(shadowprogram, "modelviewMatrix"), 1, GL_TRUE, m.m);
	gluggDrawModel(badboll, shadowprogram);

	m = Mult(cam, Mult(T(p2.x,0.1,-p2.y+1), S(1,0,1)));
    glUniformMatrix4fv(glGetUniformLocation(shadowprogram, "modelviewMatrix"), 1, GL_TRUE, m.m);
	gluggDrawModel(badboll, shadowprogram);

	m = Mult(cam, Mult(T(p3.x,0.1,-p3.y+1), S(1,0,1)));
    glUniformMatrix4fv(glGetUniformLocation(shadowprogram, "modelviewMatrix"), 1, GL_TRUE, m.m);
	gluggDrawModel(badboll, shadowprogram);

	glDisable(GL_BLEND);

	
	sgDraw();

    glutSwapBuffers();
}

void Reset()
{
	p1 = SetVec3(-4,10,0);
	p2 = SetVec3(0,10,0);
	p3 = SetVec3(4,10,0);
	v1 = SetVec3(0,0,0);
	v2 = SetVec3(0,0,0);
	v3 = SetVec3(0,0,0);
}

void myinit()
{
    glEnable(GL_DEPTH_TEST); /* enable z buffer */
    glDisable(GL_CULL_FACE);

	program = loadShaders("sphere.vert", "sphere.frag");
	shadowprogram = loadShaders("sphere.vert", "shadow.frag");

	badboll = badbollSphere(6, 10, 20, 1);
	
	// Make floor
	gluggBegin(GLUGG_QUADS);
	gluggNormal(0,1,0);
	gluggColor(0.5, 1, 0.6);
	gluggVertex(5, 0, 5);
	gluggVertex(5, 0, -15);
	gluggVertex(-5, 0, -15);
	gluggVertex(-5, 0, 5);
	floorModel = gluggBuildModel(1);

	// Make anvil
	gluggBegin(GLUGG_QUADS);
	gluggColor(0.5, 0.5, 0.6);
	gluggPushMatrix();
	gluggRotate(M_PI/2, 0,0,1);
	gluggTranslate(1, 1, 0);
	gluggCone(20, 1, 0.5); // Round end
	gluggPopMatrix();
	gluggRotate(M_PI/4, 0, 1, 0);
	gluggScale(1, 1, 0.5);
	gluggCylinderAlt(4, 1, 1.5, 2);
	anvil = gluggBuildModel(1);

	
	projectionMatrix = frustum(-0.5, 0.5, -0.5, 0.5, 1.0, 60.0);
	
	glUseProgram(program);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	glUseProgram(shadowprogram);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);

    glClearColor (1.0, 1.0, 1.0, 1.0);
    
    sgSetPosition(30, 30);
    sgSetTextColor(0,0,0);
    sgSetBackgroundColor(0.5, 0.5, 0.5, 0.5);
    sgSetFrameColor(0,0,0);
//    sgCreateStaticString(20, 20, "Only gravity, with drag, with wind");
    sgCreateStaticString(20, 20, "Only gravity, with variations");
    sgCreateButton(-1, -1, "Reset", Reset);

	Reset();
}


mouse(int button, int state, int x, int y)
{
	sgMouse(state, x, y);
}

mousedrag(int x, int y)
{
	sgMouseDrag(x, y);
}


int main(int argc, char **argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB | GLUT_DEPTH);
    glutInitContextVersion(3, 2);
    glutInitWindowSize(500, 500);
    glutRepeatingTimer(15);
    glutCreateWindow("Beachball 1");
    myinit();
    glutDisplayFunc(display);
    glutMouseFunc(mouse);
    glutMotionFunc(mousedrag);
    glutMainLoop();
}
