// The boat, demo 1, centered
// Drift and drag are included.
// Flat, friction free sail.
// The boat can not tilt and there is no jibing.

// uses framework Cocoa
// uses framework OpenGL
#define MAIN
#include <stdlib.h>
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils4.h"
#include "glugg.h"
#include "gluggShapes.h"
#include "SimpleGUI.h"
#include "beachball.h"

GLuint program, shadowprogram; // Shader
mat4 projectionMatrix;

#define kNumBalls 12
#define kBallDensity 3

gluggModel createBoat()
{  
  static vec3 n[7] =
  {
    vec3(0.0, 1.0, -1.0),
    vec3(1.0, 1.0, -0.5),
    vec3(1.5, 1.0, 0.0),
    vec3(1.0, 1.0, 0.5),
    vec3(0.0, 1.0, 1.0),
    vec3(-4.0, 1.0, 1.0),
    vec3(-4.0, 1.0, -1.0)
  };
  // Bottom
  static vec3 n1[7] =
  {
    vec3(-0.3, 0.0, -0.5),
    vec3(0.5, 0.0, -0.3),
    vec3(1.0, 0.0, 0.0),
    vec3(0.5, 0.0, 0.3),
    vec3(-0.3, 0.0, 0.5),
    vec3(-3.8, 0.0, 0.5),
    vec3(-3.8, 0.0, -0.5)
  };
	gluggBegin(0);
	
	gluggColor(1.0, 0.3, 0.3);
	
	// Create top
	gluggMode(GLUGG_TRIANGLE_FAN);
	gluggNormal(0,1,0);
	for (int i = 0; i < 7; i++)
		gluggVertexv(n[i]);
	gluggVertexv(n[0]);
	
	// Create sides
	gluggColor(1.0, 1.0, 1.0);
	gluggMode(GLUGG_TRIANGLE_STRIP);
// Hard-coded to get the normals OK
	gluggNormal(0,0,-1);
	gluggVertexv(n[0]);
	gluggVertexv(n1[0]);
	gluggNormal(1,0,-1);
	gluggVertexv(n[1]);
	gluggVertexv(n1[1]);
	gluggNormal(1,0,-1);
	gluggVertexv(n[2]);
	gluggVertexv(n1[2]);
	gluggNormal(1,0,1);
	gluggVertexv(n[2]);
	gluggVertexv(n1[2]);
	gluggNormal(1,0,1);
	gluggVertexv(n[3]);
	gluggVertexv(n1[3]);
	gluggNormal(0,0,1);
	gluggVertexv(n[4]);
	gluggVertexv(n1[4]);
	gluggNormal(0,0,1);
	gluggVertexv(n[5]);
	gluggVertexv(n1[5]);
	gluggNormal(-1,0,0);
	gluggVertexv(n[5]);
	gluggVertexv(n1[5]);
	gluggNormal(-1,0,0);
	gluggVertexv(n[6]);
	gluggVertexv(n1[6]);
	gluggNormal(0,0,-1);
	gluggVertexv(n[6]);
	gluggVertexv(n1[6]);
	gluggNormal(0,0,-1);
	gluggVertexv(n[0]);
	gluggVertexv(n1[0]);
	
	gluggPushMatrix();
	gluggTranslate(0,1,0);
	gluggCylinder(20, 4, 0.1);
	gluggPopMatrix();
	
	return gluggBuildModel(1);
}

gluggModel createSail()
{
	gluggBegin(GLUGG_TRIANGLES);
	gluggNormal(0,0,1);
	gluggVertex(0,1.5,0);
	gluggVertex(-3,1.5,0);
	gluggVertex(0,5,0);
	return gluggBuildModel(1);
}

gluggModel floorModel, ball, boat, sail;

vec3 ballPosition[kNumBalls*kNumBalls];
vec3 boatPosition, axisPos;
vec3 boatVelocity;
float rot;
float boatRotation, sailRotation;

float g = 9.81;
int check = 0;
int check2 = 0;
float ballMass = 1.0;
float plankMass = 5.0;
//float l = 0.0;
float windVelocity = 0.1;
//float startPosition = 0.0;
float axisPosition = 0.0;

void drawModelAt(gluggModel mdl, GLuint shader, mat4 mtx)
{
	glUseProgram(shader);
    glUniformMatrix4fv(glGetUniformLocation(shader, "modelviewMatrix"), 1, GL_TRUE, mtx.m);
	gluggDrawModel(mdl, shader);
}

void display(void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    mat4 cam = lookAt(0, 15, 20, 0, 0, -5, 0,1,0);
    
//    boat position never changes - balls move instead?

// The following will model:
//    boatRotation
//    boatVelocity
//    sailRotation
//    wind - affects boat and windTexture
//    drift and drag

	vec3 va = SetVec3(windVelocity, 0, 0) - boatVelocity; // vw - vs
	vec3 sn = SetVec3(sin(boatRotation+sailRotation), 0, cos(boatRotation+sailRotation)); // sail normal vector
	vec3 boatForward = SetVec3(cos(boatRotation), 0, -sin(boatRotation)); // sail normal vector
	if (sn.x < 0) sn = -sn; // if sail points against wind, reverse
							// More general: dot product!
	
	// Drag from the water
	// Drag should be strong sideways and weak along the boat direction!
	vec3 vforward, vside;
	SplitVector(boatVelocity, boatForward, &vforward, &vside);
	// Check whether vforward is forwards or backwards. Backwards drift should be strong.
	vec3 fdrag;
	if (dot(boatForward, vforward) < 0)
		fdrag = -vforward * 0.1 + -vside * 0.1;
	else
		fdrag = -vforward * 0.001 + -vside * 0.1;

	vec3 vn, vp;
	SplitVector(va, sn, &vn, &vp); // va split over sail normal
	// vn -> fw
	float ksail = 1.0;
	float kdrift = 0.01;
	float boatWeight = 1.0;
	float dt = 0.1;
	vec3 fw = vn * ksail; // Force due to the sail geometry
	// Split again over the forward vector
	vec3 fdrift, ffw;
	SplitVector(fw, boatForward, &ffw, &fdrift); // va split over sail normal
	fdrift = fdrift * kdrift; // How much tend the boat to drift? Drift should depend on the sideways velocity.
	ffw = ffw + fdrag + fdrift;

	boatVelocity = boatVelocity + ffw / boatWeight * dt;

//	printf("boatForward = ");printVec3(boatForward);
//	printf("sn = ");printVec3(sn);
//	printf("fw = ");printVec3(fw);
//	printf("ffw = ");printVec3(ffw);
//	printf("fdrag = ");printVec3(fdrag);
//	printf("fdrift = ");printVec3(fdrift);
//	printf("vs = ");printVec3(boatVelocity);
//	printf("\n");
    	
//	boatPosition = boatPosition + boatVelocity * dt; // *dt missing

	for (int i = 0; i < kNumBalls*kNumBalls; i++)
	{
		ballPosition[i] -= boatVelocity * dt;
		if (ballPosition[i].x > kNumBalls * kBallDensity / 2.0)
			ballPosition[i].x -= kNumBalls * kBallDensity;
		if (ballPosition[i].x < -kNumBalls * kBallDensity / 2.0)
			ballPosition[i].x += kNumBalls * kBallDensity;
		if (ballPosition[i].z > kNumBalls * kBallDensity / 2.0)
			ballPosition[i].z -= kNumBalls * kBallDensity;
		if (ballPosition[i].z < -kNumBalls * kBallDensity / 2.0)
			ballPosition[i].z += kNumBalls * kBallDensity;
	}
	
	mat4 rotm = T(axisPosition,0,0) * Ry(boatRotation) * T(-axisPosition,0,0);
	drawModelAt(boat, program, cam * T(boatPosition.x, boatPosition.y, boatPosition.z) * rotm);

//	drawModelAt(sail, program, cam * Ry(boatRotation + sailRotation));
	rotm = T(axisPosition,0,0) * Ry(boatRotation + sailRotation) * T(-axisPosition,0,0);
	drawModelAt(sail, program, cam * T(boatPosition.x, boatPosition.y, boatPosition.z) * rotm);

	for (int i = 0; i < kNumBalls*kNumBalls; i++)
		drawModelAt(ball, program, cam * T(ballPosition[i].x, ballPosition[i].y, ballPosition[i].z)*Ry(1.5));
	drawModelAt(floorModel, program, cam);

	sgDraw();

    glutSwapBuffers();
}

void Reset()
{
	boatPosition = SetVec3(0,0,0);
	boatVelocity = SetVec3(0,0,0);
//	l = 0.0; // rotational momentum
	
	for (int i = 0; i < kNumBalls; i++)
	for (int j = 0; j < kNumBalls; j++)
		ballPosition[i + j*kNumBalls] = SetVec3((i - kNumBalls/2)*kBallDensity, 0, (j - kNumBalls/2)*kBallDensity);
}

void Start()
{
}

void myinit()
{
    glEnable(GL_DEPTH_TEST); /* enable z buffer */
    glDisable(GL_CULL_FACE);

	program = loadShaders("sphere.vert", "sphere.frag");
	shadowprogram = loadShaders("sphere.vert", "shadow.frag");

//	gluggColor(0.3, 0.3, 0.4);
//	gluggColor(1.0, 0.6, 0.1);
//	gluggBegin(GLUGG_QUADS);
//	gluggSphere(10, 10, 0.25);
//	ball = gluggBuildModel(0);
	
	ball = badbollSphere(10, 10, 10, 0.25);

	
	boat = createBoat();
	sail = createSail();
	
	// Make floor
	gluggBegin(GLUGG_QUADS);
	gluggNormal(0,1,0);
	gluggColor(0.2, 0.5, 1.0);
	gluggVertex(25, 0, 25);
	gluggVertex(25, 0, -25);
	gluggVertex(-25, 0, -25);
	gluggVertex(-25, 0, 25);
	floorModel = gluggBuildModel(1);

	projectionMatrix = frustum(-0.5, 0.5, -0.5, 0.5, 1.0, 50.0);
	
	uploadMat4ToShader(program, "projectionMatrix", projectionMatrix);
	uploadMat4ToShader(shadowprogram, "projectionMatrix", projectionMatrix);

    glClearColor (0.2, 0.7, 0.2, 1.0);
    
    sgSetPosition(30, 30);
    sgSetTextColor(0,0,0);
    sgSetBackgroundColor(0.5, 0.5, 0.5, 0.5);
    sgSetFrameColor(0,0,0);
    sgCreateStaticString(20, 20, "THE BOAT");
    sgCreateDisplayFloat(-1, -1, "Wind velocity: ", &windVelocity);
	sgCreateSlider(-1, -1, 200, &windVelocity, 0.1, 1.5);
    sgCreateDisplayFloat(-1, -1, "Boat rotation: ", &boatRotation);
	sgCreateSlider(-1, -1, 200, &boatRotation, -M_PI, M_PI);
    sgCreateDisplayFloat(-1, -1, "Sail rotation: ", &sailRotation);
	sgCreateSlider(-1, -1, 200, &sailRotation, -2, 2);
    sgCreateButton(-1, -1, "Reset", Reset);
    sgCreateButton(-1, -1, "Start", Start);

	Reset();
}


void mouse(int button, int state, int x, int y)
{
	sgMouse(state, x, y);
}

void mousedrag(int x, int y)
{
	sgMouseDrag(x, y);
}


int main(int argc, char **argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitContextVersion(3, 2);
    glutInitWindowSize(500, 500);
    glutRepeatingTimer(30);
    glutCreateWindow("The boat 1");
    myinit();
    glutDisplayFunc(display);
    glutMouseFunc(mouse);
    glutMotionFunc(mousedrag);
    glutMainLoop();
}
