// The bowling balls demo applied to beach balls
// Movement affected by gravity and drag
// Two cases with different size and mass
// Unlike bowling balls, the mass and area are now proportional!

// uses framework Cocoa
// uses framework OpenGL
#define MAIN
#include <stdlib.h>
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils4.h"
#include "glugg.h"
#include "gluggShapes.h"
#include "SimpleGUI.h"
#include "beachball.h"

GLuint program, programfloor, shadowprogram; // Shader
mat4 projectionMatrix;

gluggModel beachball, beachball2, floorModel;

vec3 p1, p2, p3;
vec3 v1, v2, v3;

float g = 9.81;

void drawModelAt(gluggModel mdl, GLuint shader, mat4 mtx)
{
	glUseProgram(shader);
    glUniformMatrix4fv(glGetUniformLocation(shader, "modelviewMatrix"), 1, GL_TRUE, mtx.m);
	gluggDrawModel(mdl, shader);
}

void display(void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    mat4 cam = lookAt(0, 7, 14, 0, 4, 0, 0,1,0);
	
	GLfloat angle = glutGet(GLUT_ELAPSED_TIME) / 1000.0;

	float dt = 0.001;
	float mass = 0.1;
	float fg = -g*mass;
	float mass3 = mass / 4; // 2^2 instead of 2^3
	float area1 = M_PI;
	float area2 = area1 / 4;
	float Cd = 50; // Very high drag!
	
	float fd = Cd*area1*1*v2.y*v2.y/2; // drag Fd = CdAroV2/2, A = r2*M_PI, Cd = 0.1, ro = 1
	float fg2;
	if (v2.y > 0) fd = -fd; // Must be against the movement
	fg2 = -g*mass + fd;

	float fd3 = Cd*area2*1*v3.y*v3.y/2; // drag Fd = CdAroV2/2, A = r2*M_PI, Cd = 0.1, ro = 1
	float fg3;
	if (v3.y > 0) fd3 = -fd3; // Must be against the movement
	fg3 = -g*mass3 + fd3;

	float a = fg/mass;
	float a2 = fg2/mass;
	float a3 = fg3/mass3;

	v2 = v2 + vec3(0, a2*dt, 0);
	v3 = v3 + vec3(0, a3*dt, 0);

// 2. elephant
// 3. mouse

	p2 = p2 + v2;
	p3 = p3 + v3;

	if (p2.y < 1 && v2.y < 0)
	{
		v2.y = -v2.y;
		p2.y = 1;
	}
	if (p3.y < 0.5 && v3.y < 0)
	{
		v3.y = -v3.y;
		p3.y = 0.5;
	}
	
	drawModelAt(beachball, program, cam * T(p2.x, p2.y, p2.z));
	drawModelAt(beachball2, program, cam * T(p3.x, p3.y, p3.z));
	drawModelAt(floorModel, programfloor, cam);
	
// Shadows for both
// with "cheapness projection", move z by y
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	drawModelAt(beachball, shadowprogram, cam * T(p2.x,0.1,-p2.y+1) * S(1,0,1));
	drawModelAt(beachball2, shadowprogram, cam * T(p3.x,0.1,-p3.y+1) * S(1,0,1));

	glDisable(GL_BLEND);

	
	sgDraw();

    glutSwapBuffers();
}

void Reset()
{
	p2 = SetVec3(-2,10,0);
	p3 = SetVec3(2,10,0);
	v2 = SetVec3(0,0,0);
	v3 = SetVec3(0,0,0);
}

void myinit()
{
    glEnable(GL_DEPTH_TEST); /* enable z buffer */
    glEnable(GL_CULL_FACE);

	programfloor = loadShaders("sphere.vert", "sphere.frag");
	program = loadShaders("sphere.vert", "sphere.frag");
	shadowprogram = loadShaders("sphere.vert", "shadow.frag");

	beachball = badbollSphere(6, 10, 20, 1);
	beachball2 = badbollSphere(6, 10, 20, 0.5);
	
	// Make floor
	gluggBegin(GLUGG_QUADS);
	gluggNormal(0,1,0);
	gluggColor(0.5, 1, 0.6);
	gluggVertex(5, 0, 5);
	gluggVertex(5, 0, -15);
	gluggVertex(-5, 0, -15);
	gluggVertex(-5, 0, 5);
	floorModel = gluggBuildModel(1);
	
	projectionMatrix = frustum(-0.5, 0.5, -0.5, 0.5, 1.0, 60.0);
	
	glUseProgram(program);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	glUseProgram(programfloor);
	glUniformMatrix4fv(glGetUniformLocation(programfloor, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	glUseProgram(shadowprogram);
	glUniformMatrix4fv(glGetUniformLocation(shadowprogram, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);

    glClearColor (1.0, 1.0, 1.0, 1.0);
    
    sgSetPosition(30, 30);
    sgSetTextColor(0,0,0);
    sgSetBackgroundColor(0.5, 0.5, 0.5, 0.5);
    sgSetFrameColor(0,0,0);
    sgCreateStaticString(20, 20, "Different size and mass, high drag, shells");
    sgCreateButton(-1, -1, "Reset", Reset);

	Reset();
}

void mouse(int button, int state, int x, int y)
{
	sgMouse(state, x, y);
}

void mousedrag(int x, int y)
{
	sgMouseDrag(x, y);
}


int main(int argc, char **argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitContextVersion(3, 2);
    glutInitWindowSize(500, 500);
    glutRepeatingTimer(15);
    glutCreateWindow("Beach balls instead of bowling balls");
    myinit();
    glutDisplayFunc(display);
    glutMouseFunc(mouse);
    glutMotionFunc(mousedrag);
    glutMainLoop();
}
