// Bowling balls
// Movement affected by gravity and drag
// Two cases with different size and mass

// uses framework Cocoa
// uses framework OpenGL
#define MAIN
#include <stdlib.h>
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils4.h"
#include "glugg.h"
#include "gluggShapes.h"
#include "SimpleGUI.h"

GLuint program, programfloor, shadowprogram; // Shader
mat4 projectionMatrix;

gluggModel bowlingSphere(int aSlices, int hSlices, float radius)
{
	gluggBegin(0);
	
	gluggColor(0, 0.5, 1.0);
	gluggSphere(aSlices, hSlices, radius);
		
	return gluggBuildModel(1);
}

gluggModel bowlingball, bowlingball2, floorModel;

vec3 p1, p2;
vec3 v1, v2;

float g = 9.81;

void drawModelAt(gluggModel mdl, GLuint shader, mat4 mtx)
{
	glUseProgram(shader);
    glUniformMatrix4fv(glGetUniformLocation(shader, "modelviewMatrix"), 1, GL_TRUE, mtx.m);
	gluggDrawModel(mdl, shader);
}

void display(void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    mat4 cam = lookAt(0, 7, 14, 0, 4, 0, 0,1,0);
	
	// We decide that the radius of object 2 is half of object 1
	const float ratio = 0.5;
	const float radius1 = M_PI;
	
	float dt = 0.001;
	float mass1 = 0.1;
//	float fg = -g*mass1;
	float mass2 = mass1 * ratio * ratio * ratio;
	float area1 = radius1;
	float area2 = area1 * ratio * ratio;
	float Cd = 50; // Very high drag!
	
	float fd1 = Cd*area1*1*v1.y*v1.y/2; // drag Fd = CdAroV2/2, A = r2*M_PI, Cd = 0.1, ro = 1
	float fg1;
	if (v1.y > 0) fd1 = -fd1; // Must be against the movement
	fg1 = -g*mass1 + fd1;

	float fd2 = Cd*area2*1*v2.y*v2.y/2; // drag Fd = CdAroV2/2, A = r2*M_PI, Cd = 0.1, ro = 1
	float fg2;
	if (v2.y > 0) fd2 = -fd2; // Must be against the movement
	fg2 = -g*mass2 + fd2;

//	float a = fg/mass;
	float a1 = fg1/mass1;
	float a2 = fg2/mass2;

	v1 = v1 + vec3(0, a1*dt, 0);
	v2 = v2 + vec3(0, a2*dt, 0);

// 1. elephant
// 2. mouse

	p1 = p1 + v1;
	p2 = p2 + v2;

	if (p1.y < 1 && v1.y < 0)
	{
		v1.y = -v1.y;
		p1.y = 1;
	}
	if (p2.y < 0.5 && v2.y < 0)
	{
		v2.y = -v2.y;
		p2.y = 0.5;
	}
	
	drawModelAt(bowlingball, program, cam * T(p1.x, p1.y, p1.z));
	drawModelAt(bowlingball2, program, cam * T(p2.x, p2.y, p2.z));
	drawModelAt(floorModel, programfloor, cam);
	
// Shadows for both
// with "cheapness projection", move z by y
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	drawModelAt(bowlingball, shadowprogram, cam * T(p1.x,0.1,-p1.y+1) * S(1,0,1));
	drawModelAt(bowlingball2, shadowprogram, cam * T(p2.x,0.1,-p2.y+1) * S(1,0,1));

	glDisable(GL_BLEND);

	
	sgDraw();

    glutSwapBuffers();
}

void Reset()
{
	p1 = SetVec3(-2,10,0);
	p2 = SetVec3(2,10,0);
	v1 = SetVec3(0,0,0);
	v2 = SetVec3(0,0,0);
}

void myinit()
{
    glEnable(GL_DEPTH_TEST); /* enable z buffer */
    glEnable(GL_CULL_FACE);

	programfloor = loadShaders("sphere.vert", "sphere.frag");
	program = loadShaders("sphere.vert", "sphere.frag");
	shadowprogram = loadShaders("sphere.vert", "shadow.frag");

	bowlingball = bowlingSphere(10, 20, 1);
	bowlingball2 = bowlingSphere(10, 20, 0.5);
	
	// Make floor
	gluggBegin(GLUGG_QUADS);
	gluggNormal(0,1,0);
	gluggColor(0.5, 1, 0.6);
	gluggVertex(5, 0, 5);
	gluggVertex(5, 0, -15);
	gluggVertex(-5, 0, -15);
	gluggVertex(-5, 0, 5);
	floorModel = gluggBuildModel(1);
	
	projectionMatrix = frustum(-0.5, 0.5, -0.5, 0.5, 1.0, 60.0);
	
	glUseProgram(program);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	glUseProgram(programfloor);
	glUniformMatrix4fv(glGetUniformLocation(programfloor, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	glUseProgram(shadowprogram);
	glUniformMatrix4fv(glGetUniformLocation(shadowprogram, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);

    glClearColor (1.0, 1.0, 1.0, 1.0);
    
    sgSetPosition(30, 30);
    sgSetTextColor(0,0,0);
    sgSetBackgroundColor(0.5, 0.5, 0.5, 0.5);
    sgSetFrameColor(0,0,0);
    sgCreateStaticString(20, 20, "Different size and mass, solid, high drag");
    sgCreateButton(-1, -1, "Reset", Reset);

	Reset();
}

void mouse(int button, int state, int x, int y)
{
	sgMouse(state, x, y);
}

void mousedrag(int x, int y)
{
	sgMouseDrag(x, y);
}


int main(int argc, char **argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitContextVersion(3, 2);
    glutInitWindowSize(500, 500);
    glutRepeatingTimer(15);
    glutCreateWindow("Bowling balls");
    myinit();
    glutDisplayFunc(display);
    glutMouseFunc(mouse);
    glutMotionFunc(mousedrag);
    glutMainLoop();
}
