// Double bounce 2 - now with rotations!
// Two beach balls bouncing in box and each other
// Handles collisions according to the collisions section of the supplement.

// Rotations are set by wall collisions, and not adjusted at ball collisions.

// uses framework Cocoa
// uses framework OpenGL
#define MAIN
#include <stdlib.h>
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils4.h"
#include "glugg.h"
#include "gluggShapes.h"
#include "SimpleGUI.h"
#include "beachball.h"

GLuint program, shadowprogram; // Shader
mat4 projectionMatrix;

gluggModel badboll, floorModel;

vec3 p1, p2;
vec3 v1, v2;

float g = 9.81;

mat4 diffrot1; // Current rotation speed
mat4 rotmat1; // Current rotation
mat4 diffrot2; // Current rotation speed
mat4 rotmat2; // Current rotation
float radius1 = 1.0;
float radius2 = 1.0;

mat4 SetRotation(vec3 v, vec3 n, float radius)
{
	vec3 r = cross(v, n);
	vec3 vn, vp;
	SplitVector(v, n, &vn, &vp);
	mat4 diffrot = ArbRotate(r, -Norm(vp)/radius);
	return diffrot;
}

void TestCollision()
{
	vec3 n = p1 - p2;
	if (Norm(n) > radius1+radius2) return;
	vec3 v = v1 - v2;
	if (dot(v, n) > 0) return;
	
	// We assume that mass = radius^2
	float m1 = radius1*radius1;
	float m2 = radius2*radius2;

	vec3 v1a = v1 * (m1-m2)/(m1+m2) + v2 * 2*m2/(m1+m2);
	vec3 v2a = v1 * 2*m1/(m1+m2) + v2 * (m2-m1)/(m1+m2);
	v1 = v1a;
	v2 = v2a;
}

void TestFloor(vec3 *p1, vec3 *v1, mat4 *diffrot, float radius)
{
	// Test walls, mirror velocity but also set rotation according to velocity.
	// Note that it would be more realistic with friction.
	if (p1->y < radius && v1->y < 0)
	{
		v1->y = -v1->y;
		p1->y = radius;
		*diffrot = SetRotation(*v1, SetVector(0,1,0), radius);
	}
	if (p1->x < -5+radius && v1->x < 0)
	{
		v1->x = -v1->x;
		p1->x = -5+radius;
		*diffrot = SetRotation(*v1, SetVector(1,0,0), radius);
	}
	if (p1->x > 5-radius && v1->x > 0)
	{
		v1->x = -v1->x;
		p1->x = 5-radius;
		*diffrot = SetRotation(*v1, SetVector(-1,0,0), radius);
	}

	if (p1->z > 5-radius && v1->z > 0)
	{
		v1->z = -v1->z;
		p1->z = 5-radius;
		*diffrot = SetRotation(*v1, SetVector(0,0,-1), radius);
	}
	if (p1->z < -14+radius && v1->z < 0)
	{
		v1->z = -v1->z;
		p1->z = -14+radius;
		*diffrot = SetRotation(*v1, SetVector(0,0,1), radius);
	}
}

void drawModelAt(gluggModel mdl, GLuint shader, mat4 mtx)
{
	glUseProgram(shader);
    glUniformMatrix4fv(glGetUniformLocation(shader, "modelviewMatrix"), 1, GL_TRUE, mtx.m);
	gluggDrawModel(mdl, shader);
}

void display(void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    mat4 cam = lookAt(0, 7, 14, 0, 4, 0, 0,1,0);
	
// If we want the actual force:
//	float dt = 0.001;
//	float mass1 = radius1*radius1*0.1;
//	float mass2 = radius2*radius2*0.1;
//	float fg1 = -g*mass1;
//	float fg2 = -g*mass2;

	float dt = 0.001;
	float a = -g; // fg/mass;

	v1 = v1 + vec3(0, a*dt, 0); // Gravity only, scaled by 1000 as pseudo-time-step
	v2 = v2 + vec3(0, a*dt, 0); // Gravity only, scaled by 1000 as pseudo-time-step

	// Update the rotations by the angular velocity
	rotmat1 = diffrot1 * rotmat1;
	rotmat2 = diffrot2 * rotmat2;
	
	// Update positions
	p1 = p1 + v1;
	p2 = p2 + v2;

	TestFloor(&p1, &v1, &diffrot1, radius1);
	TestFloor(&p2, &v2, &diffrot2, radius2);
	TestCollision();
	
	// Draw the beachballs ratated by the rotmat* matrices
	drawModelAt(badboll, program, cam * T(p1.x, p1.y, p1.z) * rotmat1 * S(radius1, radius1, radius1));
	drawModelAt(badboll, program, cam * T(p2.x, p2.y, p2.z) * rotmat2 * S(radius2, radius2, radius2));
	drawModelAt(floorModel, program, cam);

// Shadows for both
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	drawModelAt(badboll, shadowprogram, cam * T(p1.x,0.1, p1.z) * S(radius1,0,radius1));
	drawModelAt(badboll, shadowprogram, cam * T(p2.x,0.1, p2.z) * S(radius2,0,radius2));
	glDisable(GL_BLEND);
	
	sgDraw();

    glutSwapBuffers();
}

float initHeight1 = 3;
float initHeight2 = 3;
float initXVelocity1 = 0.05;
float initZVelocity1 = 0.12;
float initXVelocity2 = 0.05;
float initZVelocity2 = 0.12;

void Reset()
{
	p1 = vec3(-4, initHeight1, 0);
	v1 = vec3(initXVelocity1, 0, initZVelocity1);

	p2 = vec3(4, initHeight2, 0);
	v2 = vec3(-initXVelocity2, 0, initZVelocity2);
}

void myinit()
{
    glEnable(GL_DEPTH_TEST); /* enable z buffer */
    glEnable(GL_CULL_FACE);
	
	rotmat1 = IdentityMatrix();
	diffrot1 = IdentityMatrix();
	rotmat2 = IdentityMatrix();
	diffrot2 = IdentityMatrix();
	
	program = loadShaders("sphere.vert", "sphere.frag");
	shadowprogram = loadShaders("sphere.vert", "shadow.frag");

	badboll = badbollSphere(6, 10, 20, 1.0);
	
	// Make floor
	gluggBegin(GLUGG_QUADS);
	gluggNormal(0,1,0);
	gluggColor(0.5, 1, 0.6);
	gluggVertex(5, 0, 5);
	gluggVertex(5, 0, -15);
	gluggVertex(-5, 0, -15);
	gluggVertex(-5, 0, 5);
	// Left wall
	gluggNormal(1,0,0);
	gluggColor(0.5, 1, 0.6);
	gluggVertex(-5, 0, 5);
	gluggVertex(-5, 0, -15);
	gluggVertex(-5, 10, -15);
	gluggVertex(-5, 10, 5);
	// Right wall
	gluggNormal(1,0,0); // ???
	gluggColor(0.5, 1, 0.6);
	gluggVertex(5, 0, 5);
	gluggVertex(5, 10, 5);
	gluggVertex(5, 10, -15);
	gluggVertex(5, 0, -15);
	// Far wall
	gluggNormal(0,0,1);
	gluggColor(0.5, 1, 0.6);
	gluggVertex(5, 10, -15);
	gluggVertex(-5, 10, -15);
	gluggVertex(-5, 0, -15);
	gluggVertex(5, 0, -15);
	floorModel = gluggBuildModel(1);
	
	projectionMatrix = frustum(-0.5, 0.5, -0.5, 0.5, 1.0, 60.0);
	
	glUseProgram(program);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	glUseProgram(shadowprogram);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);

    glClearColor (1.0, 1.0, 1.0, 1.0);
    
    sgSetPosition(10, 10);
    sgCreateStaticString(-1, -1, "Starting height 1");
	sgCreateSlider(-1, -1, 200, &initHeight1, 0, 10);
    sgCreateStaticString(-1, -1, "Starting X velocity 1");
	sgCreateSlider(-1, -1, 200, &initXVelocity1, -1, 1);
    sgCreateStaticString(-1, -1, "Starting Z velocity 1");
	sgCreateSlider(-1, -1, 200, &initZVelocity1, -1, 1);
    sgCreateStaticString(-1, -1, "Starting height 2");
	sgCreateSlider(-1, -1, 200, &initHeight2, 0, 10);
    sgCreateStaticString(-1, -1, "Starting X velocity 2");
	sgCreateSlider(-1, -1, 200, &initXVelocity2, -1, 1);
    sgCreateStaticString(-1, -1, "Starting Z velocity 2");
	sgCreateSlider(-1, -1, 200, &initZVelocity2, -1, 1);
    sgCreateStaticString(-1, -1, "Radius 1");
	sgCreateSlider(-1, -1, 200, &radius1, 0, 2);
    sgCreateStaticString(-1, -1, "Radius 2");
	sgCreateSlider(-1, -1, 200, &radius2, 0, 2);
    sgSetTextColor(0,0,0);
    sgSetFrameColor(0,0,0);
    sgCreateButton(-1, -1, "Reset", Reset);

	Reset();
}

void mouse(int button, int state, int x, int y)
{
	sgMouse(state, x, y);
}

void mousedrag(int x, int y)
{
	sgMouseDrag(x, y);
}


int main(int argc, char **argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitContextVersion(3, 2);
    glutInitWindowSize(500, 500);
    glutRepeatingTimer(15);
    glutCreateWindow("Double bounce 2");
    myinit();
    glutDisplayFunc(display);
    glutMouseFunc(mouse);
    glutMotionFunc(mousedrag);
    glutMainLoop();
}
