// Kinematic rotation
// A bouncing beach ball with kinematic (non-physical) rotation

// uses framework Cocoa
// uses framework OpenGL
#define MAIN
#include <stdlib.h>
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils4.h"
#include "glugg.h"
#include "gluggShapes.h"
#include "SimpleGUI.h"
#include "beachball.h"

GLuint program, shadowprogram; // Shader
mat4 projectionMatrix;

gluggModel badboll, floorModel;

vec3 p1;
vec3 v1;

float g = 9.81;

mat4 diffrot; // Current rotation speed
mat4 rotmat; // Current rotation
float radius = 1.0;

void SetRotation(vec3 v, vec3 n)
{
	vec3 r = cross(v, n);
	vec3 vn, vp;
	SplitVector(v, n, &vn, &vp);
	diffrot = ArbRotate(r, -Norm(vp)/radius);
}

void drawModelAt(gluggModel mdl, GLuint shader, mat4 mtx)
{
	glUseProgram(shader);
    glUniformMatrix4fv(glGetUniformLocation(shader, "modelviewMatrix"), 1, GL_TRUE, mtx.m);
	gluggDrawModel(mdl, shader);
}

void display(void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    mat4 cam = lookAt(0, 7, 14, 0, 4, 0, 0,1,0);
	
	float dt = 0.001;
	float mass = 0.1;
	float fg = -g*mass;
	
	float a = fg/mass;

	v1 = v1 + vec3(0, a*dt, 0); // Gravity only, scaled by 1000 as pseudo-time-step

	// Update the rotation by the angular velocity
	rotmat = diffrot * rotmat;

	// Update position
	p1 = p1 + v1;
	
	// Hits walls. Set rotation by the vecolity
	if (p1.y < radius && v1.y < 0)
	{
		v1.y = -v1.y;
		p1.y = radius;
		SetRotation(v1, vec3(0,1,0));
	}
	if (p1.x < -5+radius && v1.x < 0)
	{
		v1.x = -v1.x;
		p1.x = -5+radius;
		SetRotation(v1, vec3(1,0,0));
	}
	if (p1.x > 5-radius && v1.x > 0)
	{
		v1.x = -v1.x;
		p1.x = 5-radius;
		SetRotation(v1, vec3(-1,0,0));
	}
	if (p1.z > 5-radius && v1.z > 0)
	{
		v1.z = -v1.z;
		p1.z = 5-radius;
		SetRotation(v1, vec3(0,0,-1));
	}
	if (p1.z < -14+radius && v1.z < 0)
	{
		v1.z = -v1.z;
		p1.z = -14+radius;
		SetRotation(v1, vec3(0,0,1));
	}
	
	// Draw the beachball ratated by the rotmat matrix
	drawModelAt(badboll, program, cam * T(p1.x, p1.y, p1.z) * rotmat * S(radius, radius, radius));
	drawModelAt(floorModel, program, cam);
	
	// Shadow
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	drawModelAt(badboll, shadowprogram, cam * T(p1.x,0.1, p1.z) * S(radius,0,radius));
	glDisable(GL_BLEND);
	
	sgDraw();

    glutSwapBuffers();
}

float initHeight = 3;
float initXVelocity = 0.05;
float initZVelocity = 0.12;

void Reset()
{
	p1 = vec3(-4, initHeight, 0);
	v1 = vec3(initXVelocity, 0, initZVelocity);
}

void myinit()
{
    glEnable(GL_DEPTH_TEST); /* enable z buffer */
    glEnable(GL_CULL_FACE);
	
	rotmat = IdentityMatrix();
	diffrot = IdentityMatrix();
	
	program = loadShaders("sphere.vert", "sphere.frag");
	shadowprogram = loadShaders("sphere.vert", "shadow.frag");

	badboll = badbollSphere(6, 10, 20, 1.0);
	
	// Make floor
	gluggBegin(GLUGG_QUADS);
	gluggNormal(0,1,0);
	gluggColor(0.5, 1, 0.6);
	gluggVertex(5, 0, 5);
	gluggVertex(5, 0, -15);
	gluggVertex(-5, 0, -15);
	gluggVertex(-5, 0, 5);
	// Left wall
	gluggNormal(1,0,0);
	gluggColor(0.5, 1, 0.6);
	gluggVertex(-5, 0, 5);
	gluggVertex(-5, 0, -15);
	gluggVertex(-5, 10, -15);
	gluggVertex(-5, 10, 5);
	// Right wall
	gluggNormal(-1,0,0); // ???
	gluggColor(0.5, 1, 0.6);
	gluggVertex(5, 0, 5);
	gluggVertex(5, 10, 5);
	gluggVertex(5, 10, -15);
	gluggVertex(5, 0, -15);
	// Far wall
	gluggNormal(0,0,1);
	gluggColor(0.5, 1, 0.6);
	gluggVertex(5, 10, -15);
	gluggVertex(-5, 10, -15);
	gluggVertex(-5, 0, -15);
	gluggVertex(5, 0, -15);
	floorModel = gluggBuildModel(1);
	
	projectionMatrix = frustum(-0.5, 0.5, -0.5, 0.5, 1.0, 60.0);
	
	glUseProgram(program);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	glUseProgram(shadowprogram);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);

    glClearColor (1.0, 1.0, 1.0, 1.0);
    
    sgSetPosition(10, 10);
    sgCreateStaticString(-1, -1, "Starting height");
	sgCreateSlider(-1, -1, 200, &initHeight, 0, 10);
    sgCreateStaticString(-1, -1, "Starting X velocity");
	sgCreateSlider(-1, -1, 200, &initXVelocity, -1, 1);
    sgCreateStaticString(-1, -1, "Starting Z velocity");
	sgCreateSlider(-1, -1, 200, &initZVelocity, -1, 1);
    sgCreateStaticString(-1, -1, "Radius");
	sgCreateSlider(-1, -1, 200, &radius, 0, 2);
    sgSetTextColor(0,0,0);
    sgSetFrameColor(0,0,0);
    sgCreateButton(-1, -1, "Reset", Reset);

	Reset();
}

void mouse(int button, int state, int x, int y)
{
	sgMouse(state, x, y);
}

void mousedrag(int x, int y)
{
	sgMouseDrag(x, y);
}


int main(int argc, char **argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitContextVersion(3, 2);
    glutInitWindowSize(500, 500);
    glutRepeatingTimer(15);
    glutCreateWindow("Kinematic rotation");
    myinit();
    glutDisplayFunc(display);
    glutMouseFunc(mouse);
    glutMotionFunc(mousedrag);
    glutMainLoop();
}
