// Magnus force demo

// uses framework Cocoa
// uses framework OpenGL
#define MAIN
#include <stdlib.h>
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils4.h"
#include "glugg.h"
#include "gluggShapes.h"
#include "SimpleGUI.h"
#include "beachball.h"

GLuint program, shadowprogram; // Shader
mat4 projectionMatrix;

gluggModel badboll, floorModel;

vec3 p1;
vec3 v1;

float g = 9.81;
float dt = 0.01;

mat4 diffrot1; // Current rotation speed
mat4 rotmat1; // Current rotation
float radius1 = 1.0;
float omega; // Current rotation speed around z

mat4 SetRotation(vec3 v, vec3 n, float radius)
{
	vec3 r = cross(v, n);
	vec3 vn, vp;
	SplitVector(v, n, &vn, &vp);
	mat4 diffrot = ArbRotate(r, -Norm(vp)/radius * dt);
	omega = -Norm(vp)/radius * dt;
	return diffrot;
}

void TestFloor(vec3 *p1, vec3 *v1, mat4 *diffrot, float radius)
{
	if (p1->y < radius && v1->y < 0)
	{
		v1->y = -v1->y;
		p1->y = radius;
		*diffrot = SetRotation(*v1, vec3(0,1,0), radius);
	}
	if (p1->x < -5+radius && v1->x < 0)
	{
		v1->x = -v1->x;
		p1->x = -5+radius;
		*diffrot = SetRotation(*v1, vec3(1,0,0), radius);
	}
	if (p1->x > 5-radius && v1->x > 0)
	{
		v1->x = -v1->x;
		p1->x = 5-radius;
		*diffrot = SetRotation(*v1, vec3(-1,0,0), radius);
	}

	if (p1->z > 5-radius && v1->z > 0)
	{
		v1->z = -v1->z;
		p1->z = 5-radius;
		*diffrot = SetRotation(*v1, vec3(0,0,-1), radius);
	}
	if (p1->z < -14+radius && v1->z < 0)
	{
		v1->z = -v1->z;
		p1->z = -14+radius;
		*diffrot = SetRotation(*v1, vec3(0,0,1), radius);
	}
}

void drawModelAt(gluggModel mdl, GLuint shader, mat4 mtx)
{
	glUseProgram(shader);
    glUniformMatrix4fv(glGetUniformLocation(shader, "modelviewMatrix"), 1, GL_TRUE, mtx.m);
	gluggDrawModel(mdl, shader);
}

void display(void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    mat4 cam = lookAt(0, 7, 14, 0, 4, 0, 0,1,0);
	
//	GLfloat angle = glutGet(GLUT_ELAPSED_TIME) / 1000.0;

	float a = -g; // fg/mass;
	float mass = 1.0;
	float k = 10.0;
	vec3 magnus = vec3(0, k*omega*Norm(v1), 0);
	v1 = v1 + vec3(0, a*dt, 0) + magnus*dt/mass;

	rotmat1 = diffrot1 * rotmat1;

	p1 = p1 + v1*dt;

	TestFloor(&p1, &v1, &diffrot1, radius1);
	
	drawModelAt(badboll, program, cam * T(p1.x, p1.y, p1.z) * rotmat1 * S(radius1, radius1, radius1));
	drawModelAt(floorModel, program, cam);

// Shadow
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	drawModelAt(badboll, shadowprogram, cam * T(p1.x,0.1, p1.z) * S(radius1,0,radius1));
	glDisable(GL_BLEND);
	
	sgDraw();

    glutSwapBuffers();
}

float initHeight1 = 5;
float initXVelocity1 = 10;
float initRotation = 0.2;

void Reset()
{
	p1 = vec3(-4, initHeight1, 0);
	v1 = vec3(initXVelocity1, 0, 0);
	diffrot1 = Rz(initRotation*dt);
	omega = initRotation*dt;
}

void myinit()
{
    glEnable(GL_DEPTH_TEST); /* enable z buffer */
    glEnable(GL_CULL_FACE);
	
	rotmat1 = IdentityMatrix();
	diffrot1 = IdentityMatrix();
	diffrot1 = Rz(initRotation);
	omega = initRotation;

	program = loadShaders("sphere.vert", "sphere.frag");
	shadowprogram = loadShaders("sphere.vert", "shadow.frag");

	badboll = badbollSphere(6, 10, 20, 1.0);
	
	// Make floor
	gluggBegin(GLUGG_QUADS);
	gluggNormal(0,1,0);
	gluggColor(0.5, 1, 0.6);
	gluggVertex(5, 0, 5);
	gluggVertex(5, 0, -15);
	gluggVertex(-5, 0, -15);
	gluggVertex(-5, 0, 5);
	// Left wall
	gluggNormal(1,0,0);
	gluggColor(0.5, 1, 0.6);
	gluggVertex(-5, 0, 5);
	gluggVertex(-5, 0, -15);
	gluggVertex(-5, 10, -15);
	gluggVertex(-5, 10, 5);
	// Right wall
	gluggNormal(1,0,0); // ???
	gluggColor(0.5, 1, 0.6);
	gluggVertex(5, 0, 5);
	gluggVertex(5, 10, 5);
	gluggVertex(5, 10, -15);
	gluggVertex(5, 0, -15);
	// Far wall
	gluggNormal(0,0,1);
	gluggColor(0.5, 1, 0.6);
	gluggVertex(5, 10, -15);
	gluggVertex(-5, 10, -15);
	gluggVertex(-5, 0, -15);
	gluggVertex(5, 0, -15);
	floorModel = gluggBuildModel(1);
	
	projectionMatrix = frustum(-0.5, 0.5, -0.5, 0.5, 1.0, 60.0);
	
	glUseProgram(program);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	glUseProgram(shadowprogram);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);

    glClearColor (1.0, 1.0, 1.0, 1.0);
    
    sgSetPosition(10, 10);
    sgCreateStaticString(-1, -1, "Starting height 1");
	sgCreateSlider(-1, -1, 200, &initHeight1, 0, 10);
    sgCreateStaticString(-1, -1, "Starting X velocity 1");
	sgCreateSlider(-1, -1, 200, &initXVelocity1, 0, 20);
    sgCreateStaticString(-1, -1, "Starting rotation");
	sgCreateSlider(-1, -1, 200, &initRotation, -20, 20);
    sgSetTextColor(0,0,0);
//    sgSetBackgroundColor(0.5, 0.5, 0.5, 0.5);
    sgSetFrameColor(0,0,0);
////    sgCreateStaticString(20, 20, "Only gravity, with drag, with wind");
//    sgCreateStaticString(20, 20, "Without and with drag");
//    sgCreateStaticString(-1, -1, "Left: without drag");
//    sgCreateStaticString(-1, -1, "Right: more mass");
    sgCreateButton(-1, -1, "Reset", Reset);

	Reset();
}

void mouse(int button, int state, int x, int y)
{
	sgMouse(state, x, y);
}

void mousedrag(int x, int y)
{
	sgMouseDrag(x, y);
}


int main(int argc, char **argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitContextVersion(3, 2);
    glutInitWindowSize(500, 500);
    glutRepeatingTimer(15);
    glutCreateWindow("Magnus force");
    myinit();
    glutDisplayFunc(display);
    glutMouseFunc(mouse);
    glutMotionFunc(mousedrag);
    glutMainLoop();
}
