// The plank

// uses framework Cocoa
// uses framework OpenGL
#define MAIN
#include <stdlib.h>
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils4.h"
#include "glugg.h"
#include "gluggShapes.h"
#include "SimpleGUI.h"

GLuint program, shadowprogram; // Shader
mat4 projectionMatrix;

// Modified gluggCube
void createBox(GLfloat sizex, GLfloat sizey, GLfloat sizez)
{
  static vec3 n[6] =
  {
    vec3(-1.0, 0.0, 0.0),
    vec3(0.0, 1.0, 0.0),
    vec3(1.0, 0.0, 0.0),
    vec3(0.0, -1.0, 0.0),
    vec3(0.0, 0.0, 1.0),
    vec3(0.0, 0.0, -1.0)
  };
  static GLint faces[6][4] =
  {
    {0, 1, 2, 3},
    {3, 2, 6, 7},
    {7, 6, 5, 4},
    {4, 5, 1, 0},
    {5, 6, 2, 1},
    {7, 4, 0, 3}
  };
  vec3 v[8] =
  {
  	vec3(-sizex/2,-sizey/2,-sizez/2),
  	vec3(-sizex/2,-sizey/2, sizez/2),
  	vec3(-sizex/2, sizey/2, sizez/2),
  	vec3(-sizex/2, sizey/2,-sizez/2),
  	vec3( sizex/2,-sizey/2,-sizez/2),
  	vec3( sizex/2,-sizey/2, sizez/2),
  	vec3( sizex/2, sizey/2, sizez/2),
  	vec3( sizex/2, sizey/2,-sizez/2)
  };
  GLint i;
  
  gluggMode(GLUGG_QUADS);

  for (i = 5; i >= 0; i--)
  {
    gluggNormalv(n[i]);
    gluggVertexv(v[faces[i][0]]);
    gluggVertexv(v[faces[i][1]]);
    gluggVertexv(v[faces[i][2]]);
    gluggVertexv(v[faces[i][3]]);
  }
}

gluggModel plank, floorModel, ball, axis;

vec3 ballPosition = vec3(0.0, 0.5, 5);
vec3 plankPosition, axisPos;
vec3 ballVelocity, plankVelocity;
float rot, omega;

float g = 9.81;
int check = 0;
int check2 = 0;
float ballMass = 1.0;
float plankMass = 2.0;
float l = 0.0;
float startVelocity = 0.1;
//float startPosition = 0.0;
float axisPosition = 0.0;

void drawModelAt(gluggModel mdl, GLuint shader, mat4 mtx)
{
	glUseProgram(shader);
    glUniformMatrix4fv(glGetUniformLocation(shader, "modelviewMatrix"), 1, GL_TRUE, mtx.m);
	gluggDrawModel(mdl, shader);
}

void display(void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    mat4 cam = lookAt(0, 7, 14, 0, 4, 0, 0,1,0);
		
	ballPosition = ballPosition + ballVelocity; // *dt missing
	if (ballPosition.z < plankPosition.z + 0.5) // Handle collision
	{
		// Calc plankVelocity and omega
		float f = ballVelocity.z * ballMass;
		// plankVelocity.z = f / plankMass;
		ballVelocity.z = -ballVelocity.z; // Really force backwards
		ballPosition.z = plankPosition.z + 0.49; // a little back to avoid repeating
		// Plank center is at origin!
		float t = -(ballPosition.x-axisPosition) * f; // torque, really r x f
		l = l + t; // really *dt but this is an impulse
		// Inertia for a thin rod/plank rotating around its center is
		// 1/12ml^2
		// With an arbitrary axis, we can model this as a sum of two rods:
		// axisPosition goes from -2 to 2
		// Plank is form -2.5 to 2.5
		float d1 = 2.5 - axisPosition;
		float d2 = 2.5 + axisPosition;
		float j = 1/3.0*plankMass*d1*d1 + 1/3.0*plankMass*d2*d2;
		omega = l / j;
	}
	
	plankPosition = plankPosition + plankVelocity; // *dt missing
	rot = rot + omega;
	
	mat4 rotm = T(axisPosition,0,0) * Ry(rot) * T(-axisPosition,0,0);
	drawModelAt(plank, program, cam * T(plankPosition.x, plankPosition.y, plankPosition.z) * rotm);
	drawModelAt(ball, program, cam * T(ballPosition.x, ballPosition.y, ballPosition.z));
	drawModelAt(axis, program, cam * T(axisPosition, 0, 0));
	drawModelAt(floorModel, program, cam);

// Shadow is pointless for the plank since the plank is on the plane
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	drawModelAt(ball, shadowprogram, cam * T(ballPosition.x,0.1,ballPosition.z) * S(1,0,1));
	glDisable(GL_BLEND);
	
	sgDraw();

    glutSwapBuffers();
}

void Reset()
{
	// Ball
//	ballPosition = SetVec3(startPosition,0.5,5);
	ballPosition.z = 5;
	ballVelocity = SetVec3(0, 0, 0);
	// plank
	plankPosition = SetVec3(0,0.5,0);
	plankVelocity = SetVec3(0,0,0);
	rot = 0;
	omega = 0; // rotation speed
	l = 0.0; // rotational momentum
	// axis
//	axisPos = SetVec3(axisPosition, 0, 0);
}

void Start()
{
	ballVelocity = SetVec3(0, 0, -startVelocity);
}

void myinit()
{
    glEnable(GL_DEPTH_TEST); /* enable z buffer */
    glEnable(GL_CULL_FACE);

	program = loadShaders("sphere.vert", "sphere.frag");
	shadowprogram = loadShaders("sphere.vert", "shadow.frag");

	gluggBegin(GLUGG_QUADS);
		gluggColor(0.7, 0.6, 0.3);
		createBox(5, 1, 0.5);
	plank = gluggBuildModel(1);

	gluggBegin(GLUGG_QUADS);
		gluggColor(0.2, 0.2, 0.2);
		gluggCylinder(10, 1.1, 0.35);
	axis = gluggBuildModel(1);

	gluggColor(0.3, 0.3, 0.4);
	gluggBegin(GLUGG_QUADS);
	gluggSphere(10, 10, 0.25);
	ball = gluggBuildModel(1);

	// Make floor
	gluggBegin(GLUGG_QUADS);
	gluggNormal(0,1,0);
	gluggColor(0.5, 1, 0.6);
	gluggVertex(5, 0, 5);
	gluggVertex(5, 0, -15);
	gluggVertex(-5, 0, -15);
	gluggVertex(-5, 0, 5);
	floorModel = gluggBuildModel(1);

	projectionMatrix = frustum(-0.5, 0.5, -0.5, 0.5, 1.0, 60.0);
	
	glUseProgram(program);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	glUseProgram(shadowprogram);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);

    glClearColor (1.0, 1.0, 1.0, 1.0);
    
    sgSetPosition(30, 30);
    sgSetTextColor(0,0,0);
    sgSetBackgroundColor(0.5, 0.5, 0.5, 0.5);
    sgSetFrameColor(0,0,0);
    sgCreateStaticString(20, 20, "THE PLANK PART 2");
    sgCreateDisplayFloat(-1, -1, "Starting velocity: ", &startVelocity);
	sgCreateSlider(-1, -1, 200, &startVelocity, 0.1, 1.5);
    sgCreateDisplayFloat(-1, -1, "Starting position: ", &ballPosition.x);
	sgCreateSlider(-1, -1, 200, &ballPosition.x, -2, 2);
    sgCreateDisplayFloat(-1, -1, "Axis position: ", &axisPosition);
	sgCreateSlider(-1, -1, 200, &axisPosition, -2, 2);
    sgCreateButton(-1, -1, "Reset", Reset);
    sgCreateButton(-1, -1, "Start", Start);

	Reset();
}


void mouse(int button, int state, int x, int y)
{
	sgMouse(state, x, y);
}

void mousedrag(int x, int y)
{
	sgMouseDrag(x, y);
}


int main(int argc, char **argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitContextVersion(3, 2);
    glutInitWindowSize(500, 500);
    glutRepeatingTimer(15);
    glutCreateWindow("The plank 2");
    myinit();
    glutDisplayFunc(display);
    glutMouseFunc(mouse);
    glutMotionFunc(mousedrag);
    glutMainLoop();
}
