// Two balls
// Two beachballs, with impulse beween them

// uses framework Cocoa
// uses framework OpenGL
#define MAIN
#include <stdlib.h>
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils4.h"
#include "glugg.h"
#include "SimpleGUI.h"
#include "beachball.h"

GLuint program; // Shader
mat4 projectionMatrix;

gluggModel badboll;

vec3 p1, p2;
vec3 v1 = vec3(0,0,0), v2 = vec3(0,0,0);

float mass1 = 2, mass2 = 2;
float imp = 1.0;

void drawModelAt(gluggModel mdl, GLuint shader, mat4 mtx)
{
	glUseProgram(shader);
    glUniformMatrix4fv(glGetUniformLocation(shader, "modelviewMatrix"), 1, GL_TRUE, mtx.m);
	gluggDrawModel(mdl, shader);
}

void display(void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    mat4 cam = lookAt(0, 7, 14, 0, 4, 0, 0,1,0);
	
	p1 = p1 + v1;
	p2 = p2 + v2;
	
	glUseProgram(program);
	float m1 = sqrt(mass1);
	float m2 = sqrt(mass2);
	drawModelAt(badboll, program, cam * T(p1.x + m1 + 1, p1.y, p1.z) * S(m1, m1, m1));
	drawModelAt(badboll, program, cam * T(p2.x - m2 - 1, p2.y, p2.z) * S(m2, m2, m2));

	sgDraw();

    glutSwapBuffers();
}

void Reset()
{
	p1 = vec3(-1,1,0);
	p2 = vec3(1,1,0);
	v1 = vec3(0,0,0);
	v2 = vec3(0,0,0);
}

void Push()
{
	v1 = vec3(imp/mass1,0,0);
	v2 = vec3(-imp/mass2,0,0);
}

void myinit()
{
    glEnable(GL_DEPTH_TEST); /* enable z buffer */
    glDisable(GL_CULL_FACE);

	program = loadShaders("sphere.vert", "sphere.frag");

	badboll = badbollSphere(6, 10, 20, 1);
	
	projectionMatrix = frustum(-0.5, 0.5, -0.5, 0.5, 1.0, 60.0);
	
	glUseProgram(program);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);

    glClearColor (1.0, 1.0, 1.0, 1.0);
    
    sgSetPosition(30, 30);
    sgSetTextColor(0,0,0);
    sgSetBackgroundColor(0.5, 0.5, 0.5, 0.5);
    sgSetFrameColor(0,0,0);
    sgCreateStaticString(10, 10, "Two balls with different mass");
    sgCreateStaticString(-1, -1, "Left ball:");
    sgCreateSlider(-1, -1, 200, &mass2, 1, 10);
    sgCreateStaticString(-1, -1, "Right ball:");
    sgCreateSlider(-1, -1, 200, &mass1, 1, 10);
    sgCreateStaticString(-1, -1, "Impulse:");
    sgCreateSlider(-1, -1, 200, &imp, 0.1, 3);
    sgCreateButton(-1, -1, "Push!", Push);
    sgCreateButton(-1, -1, "Reset", Reset);

	Reset();
}

void mouse(int button, int state, int x, int y)
{
	sgMouse(state, x, y);
}

void mousedrag(int x, int y)
{
	sgMouseDrag(x, y);
}


int main(int argc, char **argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitContextVersion(3, 2);
    glutInitWindowSize(500, 500);
    glutRepeatingTimer(15);
    glutCreateWindow("Push");
    myinit();
    glutDisplayFunc(display);
    glutMouseFunc(mouse);
    glutMotionFunc(mousedrag);
    glutMainLoop();
}
