// Sliding block
// Demo of friction
// With no check for zero, we get an erronoeous behavior when the block is almost still,
// since the friction force is constant. For this case, it is enough to check for
// the force chanhing sign, which means that it should stop. However, if there are
// additional external forces, this becomes more complicated.

// uses framework Cocoa
// uses framework OpenGL
#define MAIN
#include <stdlib.h>
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils4.h"
#include "glugg.h"
#include "gluggShapes.h"
#include "SimpleGUI.h"

GLuint program, shadowprogram; // Shader
mat4 projectionMatrix;

gluggModel block, floorModel, anvil;

vec3 p1, p2, p3;
vec3 v1, v2, v3;

float g = 9.81;
int check = 0;
int check2 = 0;

void display(void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    mat4 cam = lookAt(0, 7, 14, 0, 4, 0, 0,1,0);
	
	// Solution 1: Check for zero crossings. Not a general solution!
	if (check)
	{
		float a = g / 100.0;
		if (v1.x > 0)
		{
			a = -a;
			if (-a > v1.x)
				v1.x = 0;
			else
				v1.x += a;
		}
		else
		{
			if (a > -v1.x)
				v1.x = 0;
			else
				v1.x += a;
		}
	}
	else
	// Solution 2: Lower the friction if the velocity is small so it can never "kick back"
	if (check2 && Norm(v1) < g / 100.0)
	{
		if (v1.x > 0)
			v1 = v1 + vec3(-Norm(v1), 0, 0); // Gravity only, scaled by 100 as pseudo-time-step
		else
			v1 = v1 + vec3(Norm(v1), 0, 0); // Gravity only, scaled by 100 as pseudo-time-step
	}
	else
	// Naive: Just add friction
	if (v1.x > 0)
		v1 = v1 + vec3(-g / 100.0, 0, 0); // Gravity only, scaled by 100 as pseudo-time-step
	else
		v1 = v1 + vec3(g / 100.0, 0, 0); // Gravity only, scaled by 100 as pseudo-time-step
	
	printf("%f\n", v1.x);
	
	p1 = VectorAdd(p1, v1);
	
	glUseProgram(program);
	mat4 m = cam * T(p1.x, p1.y, p1.z);
    glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, m.m);
	gluggDrawModel(block, program);
	
	glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, cam.m);
	gluggDrawModel(floorModel, program);

// Shadow is pointless since the block is on the plane
//	glEnable(GL_BLEND);
//	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
//	glUseProgram(shadowprogram);

//	m = Mult(cam, Mult(T(p1.x,0.1,-p1.y+1), S(1,0,1)));
//    glUniformMatrix4fv(glGetUniformLocation(shadowprogram, "modelviewMatrix"), 1, GL_TRUE, m.m);
//	gluggDrawModel(block, shadowprogram);

//	glDisable(GL_BLEND);

	
	sgDraw();

    glutSwapBuffers();
}

float startVelocity = 0.4;

void Reset()
{
	p1 = vec3(-4,1,0);
	v1 =vec3(startVelocity,0,0);
}

void myinit()
{
    glEnable(GL_DEPTH_TEST); /* enable z buffer */
    glDisable(GL_CULL_FACE);

	program = loadShaders("sphere.vert", "sphere.frag");
	shadowprogram = loadShaders("sphere.vert", "shadow.frag");

	gluggColor(1, 1, 0.6);
	gluggBegin(GLUGG_QUADS);
		gluggCube(2);
	block = gluggBuildModel(1);
	
	// Make floor
	gluggBegin(GLUGG_QUADS);
	gluggNormal(0,1,0);
	gluggColor(0.5, 1, 0.6);
	gluggVertex(5, 0, 5);
	gluggVertex(5, 0, -15);
	gluggVertex(-5, 0, -15);
	gluggVertex(-5, 0, 5);
	floorModel = gluggBuildModel(1);

	projectionMatrix = frustum(-0.5, 0.5, -0.5, 0.5, 1.0, 60.0);
	
	glUseProgram(program);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	glUseProgram(shadowprogram);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);

    glClearColor (1.0, 1.0, 1.0, 1.0);
    
    sgSetPosition(30, 30);
    sgSetTextColor(0,0,0);
    sgSetBackgroundColor(0.5, 0.5, 0.5, 0.5);
    sgSetFrameColor(0,0,0);
    sgCreateStaticString(20, 20, "Friction");
    sgCreateDisplayFloat(-1, -1, "Starting velocity: ", &startVelocity);
	sgCreateSlider(-1, -1, 200, &startVelocity, 0.1, 1.5);
    sgCreateDisplayFloat(-1, -1, "Current velocity: ", &v1.x);
    sgCreateCheckBox(-1, -1, "Zero crossing check", &check);
    sgCreateCheckBox(-1, -1, "Low speed check", &check2);
    sgCreateButton(-1, -1, "Reset", Reset);

	Reset();
}


void mouse(int button, int state, int x, int y)
{
	sgMouse(state, x, y);
}

void mousedrag(int x, int y)
{
	sgMouseDrag(x, y);
}


int main(int argc, char **argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitContextVersion(3, 2);
    glutInitWindowSize(500, 500);
    glutRepeatingTimer(15);
    glutCreateWindow("Bad and good friction");
    myinit();
    glutDisplayFunc(display);
    glutMouseFunc(mouse);
    glutMotionFunc(mousedrag);
    glutMainLoop();
}
