// Sliding block
// Demo of friction
// With no check for zero, we get an erronoeous behavior when the block is almost still,
// since the friction force is constant. For this case, it is enough to check for
// the force chanhing sign, which means that it should stop. However, if there are
// additional external forces, this becomes more complicated.

// uses framework Cocoa
// uses framework OpenGL
#define MAIN
#include <stdlib.h>
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils4.h"
#include "glugg.h"
#include "gluggShapes.h"
#include "SimpleGUI.h"

GLuint program, shadowprogram; // Shader
mat4 projectionMatrix;

gluggModel block, floorModel, anvil;

vec3 p1, p2, p3;
vec3 v1, v2, v3;

float g = 9.81;
int check = 0;
int check2 = 0;
float slope = 0.0;

void display(void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    mat4 cam = lookAt(0, 7, 14, 0, 4, 0, 0,1,0);
	
//	GLfloat angle = glutGet(GLUT_ELAPSED_TIME) / 1000.0;
	float mass = 1.0;
	float dt = 0.01;
	float mys = 0.9; // static coefficient
	float myd = 0.6; // dynamic coefficient
	
	// Resulting force. We start by adding effect from slope,
	// sideways only since the perpendicular composant will just be opposed by an equal one from the surface.
//	vec3 f = {sin(angle),0,0};

	// No signs are needed here since everything happens in the known slope and I can work on the magnitudes only.
	
//	float f = g*sin(slope)/100.0;
	
	// TOTALLY WRONG BUT WHY?
	
	float fg = g*mass;
	float fpush = fg*sin(slope);
	float fr;
	float fgn = fg*cos(slope);
	float fsmax = fgn * mys;
	float fd = fgn * myd;
//	if (v1.x != 0) // Dynamic
	if (fabs(v1.x) > 0.01) // Dynamic
		fr = fmin(v1.x*mass/dt, fd); // Don't kick back
//		fr = fd; // Will kick back
	else // Static
		fr = fmax(fpush, fsmax);
	float f = fpush - fr;
	v1.x += f/mass*dt;
	
	// Solution 1: Check for zero crossings. Not a general solution!
	// BAD - did only add problems
//	if (check)
//	{
//		float a = f/mass*dt;
//		if (v1.x > 0)
//		{
////			a = -a;
//			if (a < v1.x)
//				v1.x = 0;
//			else
//				v1.x += a;
//		}
//		else
//		{
//			if (a > -v1.x)
//				v1.x = 0;
//			else
//				v1.x += a;
//		}
//	}
//	else
		v1.x += f/mass*dt; // ???
		
		
//	else
	// Solution 2: Lower the friction if the velocity is small so it can never "kick back"
//	if (check2 && Norm(v1) < g / 100.0)
//	{
//		if (v1.x > 0)
//			f += -Norm(v1);
//		else
//			f += Norm(v1);
//	}
//	else
//	// Naive: Just add friction
//	if (v1.x > 0)
//		f += -g / 100.0; // Gravity only, scaled by 100 as pseudo-time-step
//	else
//		f += g / 100.0; // Gravity only, scaled by 100 as pseudo-time-step
//	
//	v1.x += f;
	
	printf("%f\n", v1.x);
	
	p1 = p1 + v1;
	
	glUseProgram(program);
	mat4 m = Rz(-slope) * cam * T(p1.x, p1.y, p1.z);
    glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, m.m);
	gluggDrawModel(block, program);
	
	m = Rz(-slope) * cam;
	glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, m.m);
	gluggDrawModel(floorModel, program);

// Shadow is pointless since the block is on the plane
//	glEnable(GL_BLEND);
//	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
//	glUseProgram(shadowprogram);

//	m = Mult(cam, Mult(T(p1.x,0.1,-p1.y+1), S(1,0,1)));
//    glUniformMatrix4fv(glGetUniformLocation(shadowprogram, "modelviewMatrix"), 1, GL_TRUE, m.m);
//	gluggDrawModel(block, shadowprogram);

//	glDisable(GL_BLEND);

	
	sgDraw();

    glutSwapBuffers();
}

float startVelocity = 0.4;

void Reset()
{
	p1 = vec3(-4,1,0);
	v1 = vec3(startVelocity,0,0);
}

void myinit()
{
    glEnable(GL_DEPTH_TEST); /* enable z buffer */
    glDisable(GL_CULL_FACE);

	program = loadShaders("sphere.vert", "sphere.frag");
	shadowprogram = loadShaders("sphere.vert", "shadow.frag");

	gluggColor(1, 1, 0.6);
	gluggBegin(GLUGG_QUADS);
		gluggCube(2);
	block = gluggBuildModel(1);
	
	// Make floor
	gluggBegin(GLUGG_QUADS);
	gluggNormal(0,1,0);
	gluggColor(0.5, 1, 0.6);
	gluggVertex(5, 0, 5);
	gluggVertex(5, 0, -15);
	gluggVertex(-5, 0, -15);
	gluggVertex(-5, 0, 5);
	floorModel = gluggBuildModel(1);

	projectionMatrix = frustum(-0.5, 0.5, -0.5, 0.5, 1.0, 60.0);
	
	glUseProgram(program);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	glUseProgram(shadowprogram);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);

    glClearColor (1.0, 1.0, 1.0, 1.0);
    
    sgSetPosition(30, 30);
    sgSetTextColor(0,0,0);
    sgSetBackgroundColor(0.5, 0.5, 0.5, 0.5);
    sgSetFrameColor(0,0,0);
    sgCreateStaticString(20, 20, "Friction example");
	sgCreateSlider(-1, -1, 200, &startVelocity, -1.5, 1.5);
    sgCreateDisplayFloat(-1, -1, "Starting velocity: ", &startVelocity);
    sgCreateDisplayFloat(-1, -1, "Current velocity: ", &v1.x);
	sgCreateSlider(-1, -1, 200, &slope, 0.0, M_PI/2.0);
    sgCreateDisplayFloat(-1, -1, "Slope: ", &slope);
    sgCreateCheckBox(-1, -1, "Zero crossing check", &check);
    sgCreateCheckBox(-1, -1, "Low speed check", &check2);
    sgCreateButton(-1, -1, "Reset", Reset);

	Reset();
}


void mouse(int button, int state, int x, int y)
{
	sgMouse(state, x, y);
}

void mousedrag(int x, int y)
{
	sgMouseDrag(x, y);
}


int main(int argc, char **argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitContextVersion(3, 2);
    glutInitWindowSize(500, 500);
    glutRepeatingTimer(15);
    glutCreateWindow("Bad and good friction");
    myinit();
    glutDisplayFunc(display);
    glutMouseFunc(mouse);
    glutMotionFunc(mousedrag);
    glutMainLoop();
}
