// Sproing, a pretty minimal (over-simplified) 2D spring system
// This is adapted from an older demo which is why the coordinates are large (it used to be screen coordinates).

// uses framework Cocoa
// uses framework OpenGL
#define MAIN
#include <stdlib.h>
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils4.h"
#include "glugg.h"
#include "gluggShapes.h"
#include "SimpleGUI.h"
#include "beachball.h"


GLuint program, shadowprogram; // Shader
mat4 projectionMatrix;

gluggModel badboll, floorModel; //, shell;


typedef struct Rect
{
	float left, top, right, bottom;
} Rect;

Rect ram, box;

typedef struct Partikel
{
	vec2 p, oldp;
	vec2 v;
	vec2 a;
	float m;
	vec2 f;
} Partikel, *PartikelPtr;

typedef struct Fjader
{
	PartikelPtr nod1, nod2;
	float length;
} Fjader;

enum
{
	kLappSizeH = 10,
	kLappSizeV = 5,
	kBounce = 1,
	kPlast = 0,
	kNodeDist = 10
};

int euler = 1; // false = Verlet
float startHeight = 30;
float startVelocity = -1.0;

struct Partikel lapp[kLappSizeH+1][kLappSizeV+1];
struct Fjader fjFram[kLappSizeH+1][kLappSizeV+1], fjSned[kLappSizeH][kLappSizeV], fjNed[kLappSizeH+1][kLappSizeV+1], fjSnedBak[kLappSizeH+1][kLappSizeV+1];
int h, v;

float Dot (vec2 a, vec2 b)
{
	return a.x * b.x + a.y * b.y;
}

inline
vec2 operator+(const vec2 &a, const vec2 &b) // vec2+vec2
{
	vec2 r;
	r.x = a.x + b.x;
	r.y = a.y + b.y;
	return r;
}

inline
vec2 operator-(const vec2 &a, const vec2 &b) // vec2-vec2
{
	vec2 r;
	r.x = a.x - b.x;
	r.y = a.y - b.y;
	return r;
}

inline
vec2 operator*(const vec2 &a, double b) // vec2 * scalar
{
	vec2 r;
	r.x = a.x*b;
	r.y = a.y*b;
	return r;
//	return Scale(a, b);
}

inline
vec2 operator*(double a, const vec2 &b) // vec2 * scalar
{
	vec2 r;
	r.x = b.x*a;
	r.y = b.y*a;
	return r;
}

inline
vec2 operator/(const vec2 &a, double b) // vec2 * scalar
{
	vec2 r;
	r.x = a.x/b;
	r.y = a.y/b;
	return r;
}

float VectorLength (vec2 a) // Norm
{
	return sqrt(a.x * a.x + a.y * a.y);
}

vec2 NormalizeVec2 (vec2 a)
{
	float r;
	r = sqrt(a.x * a.x + a.y * a.y);
	return (a / r);
}

Fjader SkapaFjader (int h1, int v1, int h2, int v2)
{
	Fjader f;

	f.nod1 = NULL;
	f.nod2 = NULL;
	f.length = VectorLength(lapp[h1][v1].p - lapp[h2][v2].p);
// Ensure h2, v2 are in the array. h1, v1 are assumed to be.
	if (h2 >= 1)
		if (h2 <= kLappSizeH)
			if (v2 >= 1)
				if (v2 <= kLappSizeV)
				{
					f.nod1 = &lapp[h1][v1];
					f.nod2 = &lapp[h2][v2];
				}
	return f;
}

float ks = 0.431;
float kd = 0.02;
float randomness = 1.0;

// Calculate spring forces, apply to nodes
void CalcForce (Fjader f)
{
	vec2 a, b, c, d;
	float len;
	vec2 vd;
	float ddd;

	a = f.nod1->p;
	b = f.nod2->p;

	c = a - b;
	len = VectorLength(c);
	d = NormalizeVec2(c);

	vd = f.nod1->v - f.nod2->v;
	ddd = Dot(vd, d);

// Without dampening:
//		d = Scale(d, (len - f.length) * ks);
// With dampening
	d = d * ((len - f.length) * ks + kd * ddd);

	f.nod1->f = f.nod1->f - d;
	f.nod2->f = f.nod2->f + d;
};

float Rand (float arg)
{
	return ((float)rand() / RAND_MAX - 0.5) * arg;
};

vec2 nextold, vv;
int animation = 0;
int numSprings = 8;
//int drawShell = 1;
//int drawParticles = 1;

void gluggVertex2D(vec2 v)
{
	gluggVertex(v.x, v.y, 5.0);
}

vec3 cross2D(vec2 a, vec2 b)
{
	return cross(SetVec3(a.x,a.y,0), SetVec3(b.x,b.y,0));
}

// The shell is not interesting in 2D since it is always flat
//void BuildShell()
//{
//	gluggBegin(GLUGG_QUADS);
//	
//	// Z sides
//	gluggColor(1, 1, 0.5); // Light yellow
//	for (int x = 1; x <= kLappSizeH-1; x++)
//		for (int y = 1; y <= kLappSizeV-1; y++)
//		{
//			gluggNormalv(cross2D(lapp[x][y+1].p - lapp[x][y].p, lapp[x+1][y].p - lapp[x][y].p));
//			gluggVertex2D(lapp[x][y].p);
//			gluggVertex2D(lapp[x+1][y].p);
//			gluggVertex2D(lapp[x+1][y+1].p);
//			gluggVertex2D(lapp[x][y+1].p);
//		}
////	gluggNormalv(cross(p212-p112, p122 - p112));
//	gluggColor(1, 0, 0.5); // 
//	for (int x = 1; x <= kLappSizeH-1; x++)
//		for (int y = 1; y <= kLappSizeV-1; y++)
//		{
//			gluggNormalv(cross2D(lapp[x+1][y].p - lapp[x][y].p, lapp[x][y+1].p - lapp[x][y].p));
//			gluggVertex2D(lapp[x][y+1].p);
//			gluggVertex2D(lapp[x+1][y+1].p);
//			gluggVertex2D(lapp[x+1][y].p);
//			gluggVertex2D(lapp[x][y].p);
//		}

//	shell = gluggBuildModel(1);
//}

void Animate()
{
	if (animation)
	{
	
// Zero forces, add gravity.
	for (h = 1; h <= kLappSizeH; h++)
		for (v = 1; v <= kLappSizeV; v++)
			{
				lapp[h][v].f.x = 0;
				lapp[h][v].f.y = -0.03; //gravitation
			};

// Create forces from positions
	for (h = 1; h <= kLappSizeH; h++)
		for (v = 1; v <= kLappSizeV; v++)
			{
				if (fjFram[h][v].nod1 != NULL)
					CalcForce(fjFram[h][v]);
				if (fjSned[h][v].nod1 != NULL)
					CalcForce(fjSned[h][v]);
				if (numSprings == 6 || numSprings == 8)
					if (fjNed[h][v].nod1 != NULL)
						CalcForce(fjNed[h][v]);
				if (numSprings == 8)
					if (fjSnedBak[h][v].nod1 != NULL)
						CalcForce(fjSnedBak[h][v]);
			};

// for all notes in "lapp"
	for (h = 1; h <= kLappSizeH; h++)
		for (v = 1; v <= kLappSizeV; v++)
		{
			nextold = lapp[h][v].p;

			if (euler)
//*** Model 1: Euler integration
			{
// Calculate acceleration,
				lapp[h][v].a = lapp[h][v].f / lapp[h][v].m;

// Add to velocity
				lapp[h][v].v = lapp[h][v].v + lapp[h][v].a;

// Add velocity to position
				lapp[h][v].p = lapp[h][v].p + lapp[h][v].v;
			}
			else
			{
//*** Model 2: Verlet integration

// Calculate acceleration,
				lapp[h][v].a = lapp[h][v].f / lapp[h][v].m;
				
				vv = lapp[h][v].p - lapp[h][v].oldp;
				lapp[h][v].p = lapp[h][v].p + vv + lapp[h][v].a;

// This step needs lapp[h][v].oldp
			};

// Limit position to the frame
			if (lapp[h][v].p.x < ram.left)
				{
					lapp[h][v].p.x = ram.left;
					if (kBounce)
						lapp[h][v].v.x = fabs(lapp[h][v].v.x);
					else if (kPlast)
						lapp[h][v].v.x = 0;
				};
			if (lapp[h][v].p.x > ram.right)
				{
					lapp[h][v].p.x = ram.right;
					if (kBounce)
						lapp[h][v].v.x = -fabs(lapp[h][v].v.x);
					else if (kPlast)
						lapp[h][v].v.x = 0;
				};
			if (lapp[h][v].p.y < ram.top)
				{
					lapp[h][v].p.y = ram.top;
					if (kBounce)
						lapp[h][v].v.y = fabs(lapp[h][v].v.y);
					else if (kPlast)
						lapp[h][v].v.y = 0;
				};
			if (lapp[h][v].p.y > ram.bottom)
				{
					lapp[h][v].p.y = ram.bottom;
					if (kBounce)
						lapp[h][v].v.y = -fabs(lapp[h][v].v.y);
					else if (kPlast)
						lapp[h][v].v.y = 0;
				};
			lapp[h][v].oldp = nextold;
		};
	
//		gluggDisposeModel(shell);
//		BuildShell();	
	}; // if (animation
}

#define ballRadius 5

void drawModelAt(gluggModel mdl, GLuint shader, mat4 mtx)
{
	glUseProgram(shader);
    glUniformMatrix4fv(glGetUniformLocation(shader, "modelviewMatrix"), 1, GL_TRUE, mtx.m);
	gluggDrawModel(mdl, shader);
}

void DrawSpringSystem(mat4 cam)
{
	// Draw particles:
		for (h = 1; h <= kLappSizeH; h++)
			for (v = 1; v <= kLappSizeV; v++)
				{
					// Draw small beachball at lapp[h][v].p
					drawModelAt(badboll, program, cam * T(lapp[h][v].p.x, lapp[h][v].p.y, 5) * S(ballRadius, ballRadius, ballRadius));
				};

		// Draw shadows using stencil
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		
		glEnable(GL_STENCIL_TEST);
		glClearStencil(0); // Specify value for glClear to set stencil to.
		glClear(GL_STENCIL_BUFFER_BIT);
		glStencilOp(GL_KEEP, GL_KEEP, GL_INCR);
		glStencilFunc(GL_EQUAL, 0, 0xFFFFFFFF);

		for (h = 1; h <= kLappSizeH; h++)
		for (v = 1; v <= kLappSizeV; v++)
		{
			drawModelAt(badboll, shadowprogram, cam * T(lapp[h][v].p.x, 1.1-ballRadius, 5) * S(ballRadius, 0, ballRadius));
		}
		glDisable(GL_BLEND);
		glDisable(GL_STENCIL_TEST);

//	if (drawShell)
//		drawModelAt(shell, program, cam);
};

Rect MakeRect(float left, float top, float right, float bottom)
{
	Rect r;
	r.left = left;
	r.top = top;
	r.right = right;
	r.bottom = bottom;
	return r;
}

void initSystem()
{
// Initialize. Zero and place.
	for (h = 1; h <= kLappSizeH; h++)
		for (v = 1; v <= kLappSizeV; v++)
		{
			lapp[h][v].p.x = 0 + kNodeDist * h;
			lapp[h][v].p.y = 0 + kNodeDist * v + startHeight;
			lapp[h][v].v.x = (Rand(1.0) - 1) * randomness + startVelocity;
			lapp[h][v].v.y = (Rand(1.0) - 0.5) * randomness;
			lapp[h][v].m = 1;
			lapp[h][v].oldp = lapp[h][v].p - lapp[h][v].v;
		};
// Create springs
	for (h = 1; h <= kLappSizeH; h++)
		for (v = 1; v <= kLappSizeV; v++)
		{
			fjFram[h][v] = SkapaFjader(h, v, h + 1, v);
			fjSned[h][v] = SkapaFjader(h, v, h + 1, v + 1);
			fjNed[h][v] = SkapaFjader(h, v, h, v + 1);
			fjSnedBak[h][v] = SkapaFjader(h, v, h - 1, v + 1);
		};
}


float g = 9.81;

void display(void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    mat4 cam = lookAt(150, 100, 450,   150, 100, 0,   0,1,0);
	
	Animate();
	DrawSpringSystem(cam);

	drawModelAt(floorModel, program, cam);
	
	sgDraw();

    glutSwapBuffers();
}

void Start()
{
	animation = 1;
}

void Reset()
{
	initSystem();
	animation = 0;
//	BuildShell();	
}

void myinit()
{
    glEnable(GL_DEPTH_TEST); /* enable z buffer */
    glDisable(GL_CULL_FACE);

	program = loadShaders("sphere.vert", "sphere.frag");
	shadowprogram = loadShaders("sphere.vert", "shadow.frag");

	badboll = badbollSphere(6, 10, 20, 1.0);

	ram = MakeRect(1, 1, 300, 300);
	Rect ramplus = MakeRect(1-ballRadius, 1-ballRadius, 300+ballRadius, 300+ballRadius);

	// Make floor
	gluggBegin(GLUGG_QUADS);
	gluggNormal(0,1,0);
	gluggColor(0.5, 1, 0.6);
	gluggVertex(ramplus.right, ramplus.top, 200); // The top is the bottom... for historical reasons
	gluggVertex(ramplus.right, ramplus.top, -50);
	gluggVertex(ramplus.left, ramplus.top, -50);
	gluggVertex(ramplus.left, ramplus.top, 200);
	// Left wall
	gluggNormal(1,0,0);
//	gluggColor(0.3, 0.8, 0.4);
	gluggVertex(ramplus.left, ramplus.top, 200);
	gluggVertex(ramplus.left, ramplus.top, -50);
	gluggVertex(ramplus.left, ramplus.bottom, -50);
	gluggVertex(ramplus.left, ramplus.bottom, 200);
	// Right wall
	gluggNormal(-1,0,0); // ???
//	gluggColor(0.5, 1, 0.6);
	gluggVertex(ramplus.right, ramplus.bottom, 200);
	gluggVertex(ramplus.right, ramplus.top, 200);
	gluggVertex(ramplus.right, ramplus.top, -50);
	gluggVertex(ramplus.right, ramplus.bottom, -50);
	// Far wall
	gluggNormal(0,0,1);
//	gluggColor(0.4, 0.9, 0.5);
	gluggVertex(ramplus.right, ramplus.top, -50);
	gluggVertex(ramplus.left, ramplus.top, -50);
	gluggVertex(ramplus.left, ramplus.bottom, -50);
	gluggVertex(ramplus.right, ramplus.bottom, -50);
	floorModel = gluggBuildModel(1);
	
	projectionMatrix = frustum(-0.5, 0.5, -0.5, 0.5, 1.0, 1000.0);
	
	glUseProgram(program);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	glUseProgram(shadowprogram);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);

    glClearColor (0.5, 0.8, 1.0, 1.0);
    
    sgSetPosition(10, 10);
    sgSetTextColor(0,0,0);
    sgSetBackgroundColor(0.5, 0.5, 0.5, 0.5);
    sgSetFrameColor(0,0,0);

    sgCreateStaticString(-1, -1, "SPROING");
    sgCreateDisplayFloat(-1, -1, "Starting height: ", &startHeight);
    sgCreateSlider(-1, -1, 200, &startHeight, 0, 200);
    sgCreateDisplayFloat(-1, -1, "X velocity: ", &startVelocity);
    sgCreateSlider(-1, -1, 200, &startVelocity, -5, 5);
    sgCreateDisplayFloat(-1, -1, "Spring constant: ", &ks);
    sgCreateSlider(-1, -1, 200, &ks, 0, 0.5);
    sgCreateDisplayFloat(-1, -1, "Dampening factor: ", &kd);
    sgCreateSlider(-1, -1, 200, &kd, 0, 0.1);
    sgCreateDisplayFloat(-1, -1, "Randomness: ", &randomness);
    sgCreateSlider(-1, -1, 200, &randomness, 0, 5);
    
	sgCreateRadio(-1, -1, "4 springs", &numSprings, 4);
	sgCreateRadio(-1, -1, "6 springs", &numSprings, 6);
	sgCreateRadio(-1, -1, "8 springs", &numSprings, 8);
	
    sgCreateCheckBox(-1, -1, "Euler", &euler);
    sgCreateButton(-1, -1, "Start", Start);
	sgCreateButton(-1, -1, "Reset", Reset);

	initSystem();
//	BuildShell();	

	Reset();
}

void Key(unsigned char key, int x, int y)
{
	if (key == ' ')
		animation = !animation;
	if (key == 'e')
		euler = 1;
	if (key == 'v')
		euler = 0;
	if (key == 'r')
		Reset();
//	if (key == 's')
//		drawShell = !drawShell;
//	if (key == 'p')
//		drawParticles = !drawParticles;
};

void mouse(int button, int state, int x, int y)
{
	sgMouse(state, x, y);
}

void mousedrag(int x, int y)
{
	sgMouseDrag(x, y);
}


int main(int argc, char **argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH | GLUT_STENCIL);
    glutInitContextVersion(3, 2);
    glutInitWindowSize(600, 600);
    glutRepeatingTimer(30);
    glutCreateWindow("Sproing");
    myinit();
    glutDisplayFunc(display);
    glutMouseFunc(mouse);
    glutMotionFunc(mousedrag);
    glutKeyboardFunc(Key);
    glutMainLoop();
}

