// Sproing, a pretty minimal (over-simplified) 3D spring system
// This is adapted from an older demo which is why the coordinates are large (it used to be screen coordinates).
// Can optionally show a "shell" of the cube and add random "kicks" upwards.

// CURRENT PROBLEMS:
// 1) All the springs are multiple (double). This makes it unnecessarily performance heavy.
// 2) Options for fewer springs are not implemented.

// uses framework Cocoa
// uses framework OpenGL
#define MAIN
#include <stdlib.h>
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils4.h"
#include "glugg.h"
#include "gluggShapes.h"
#include "SimpleGUI.h"
#include "beachball.h"


GLuint program, shadowprogram; // Shader
mat4 projectionMatrix;

gluggModel badboll, floorModel, shell;


typedef struct Rect
{
	float left, top, near, right, bottom, far;
} Rect;

Rect ram, box;

typedef struct Partikel
{
	vec3 p, oldp;
	vec3 v;
	vec3 a;
	float m;
	vec3 f;
} Partikel, *PartikelPtr;

typedef struct Fjader
{
	PartikelPtr nod1, nod2;
	float length;
} Fjader;

enum
{
	kLappSizeX = 5,
	kLappSizeY = 5,
	kLappSizeZ = 5,
	kBounce = 1,
	kPlast = 0,
	kNodeDist = 20
};

int euler = 1; // false = Verlet
float startHeight = 30;

struct Partikel lapp[kLappSizeX+1][kLappSizeY+1][kLappSizeZ+1];
struct Fjader spring[kLappSizeX+1][kLappSizeY+1][kLappSizeZ+1][3][3][3];

Fjader SkapaFjader (int x1, int y1, int z1, int x2, int y2, int z2)
{
	Fjader f;

	f.nod1 = NULL;
	f.nod2 = NULL;
	f.length = Norm(lapp[x1][y1][z1].p - lapp[x2][y2][z2].p);
// Ensure x2, y2 are in the array. x1, y1 are assumed to be.
	if (x2 >= 1)
		if (x2 <= kLappSizeX)
			if (y2 >= 1)
				if (y2 <= kLappSizeY)
					if (z2 >= 1)
						if (z2 <= kLappSizeZ)
						{
							f.nod1 = &lapp[x1][y1][z1];
							f.nod2 = &lapp[x2][y2][z2];
						}
	return f;
}

float ks = 0.08; // Was 0.431, but all springs are multple springs so it must be much lower
float kd = 0.02;

// Calculate spring forces, apply to nodes
void CalcForce (Fjader f)
{
	vec3 a, b, c, d;
	float len;
	vec3 vd;
	float ddd;

	a = f.nod1->p;
	b = f.nod2->p;

	c = a - b;
	len = Norm(c);
	d = Normalize(c);

	vd = f.nod1->v - f.nod2->v;
	ddd = dot(vd, d);

// Without dampening:
//		d = Scale(d, (len - f.length) * ks);
// With dampening
	d = d * ((len - f.length) * ks + kd * ddd);

	f.nod1->f = f.nod1->f - d;
	f.nod2->f = f.nod2->f + d;
};

float Rand (float arg)
{
	return ((float)rand() / RAND_MAX - 0.5) * arg;
};

vec3 nextold, vv;
int animation = 0;
int numSprings = 8;

void Key(char key)
{
	if (key == ' ')
		animation = !animation;
	if (key == 'e')
		euler = 1;
	if (key == 'v')
		euler = 0;
};

void BuildShell()
{
	gluggBegin(GLUGG_QUADS);
	
	// Z sides
	gluggColor(1, 1, 0.5); // Light yellow
	for (int x = 1; x <= kLappSizeX-1; x++)
		for (int y = 1; y <= kLappSizeY-1; y++)
		{
			gluggNormalv(cross(lapp[x][y+1][1].p - lapp[x][y][1].p, lapp[x+1][y][1].p - lapp[x][y][1].p));
			gluggVertexv(lapp[x][y][1].p);
			gluggVertexv(lapp[x+1][y][1].p);
			gluggVertexv(lapp[x+1][y+1][1].p);
			gluggVertexv(lapp[x][y+1][1].p);
		}
	gluggColor(1, 0, 0.5); // 
	
	for (int x = 1; x <= kLappSizeX-1; x++)
		for (int y = 1; y <= kLappSizeY-1; y++)
		{
			gluggNormalv(cross(lapp[x+1][y][kLappSizeZ].p - lapp[x][y][kLappSizeZ].p, lapp[x][y+1][kLappSizeZ].p - lapp[x][y][kLappSizeZ].p));
			gluggVertexv(lapp[x][y+1][kLappSizeZ].p);
			gluggVertexv(lapp[x+1][y+1][kLappSizeZ].p);
			gluggVertexv(lapp[x+1][y][kLappSizeZ].p);
			gluggVertexv(lapp[x][y][kLappSizeZ].p);
		}

	// X sides
	gluggColor(0, 0, 1.0);
	for (int y = 1; y <= kLappSizeY-1; y++)
		for (int z = 1; z <= kLappSizeZ-1; z++)
		{
			gluggNormalv(cross(lapp[1][y][z+1].p - lapp[1][y][z].p, lapp[1][y+1][z].p - lapp[1][y][z].p));
			gluggVertexv(lapp[1][y][z].p);
			gluggVertexv(lapp[1][y][z+1].p);
			gluggVertexv(lapp[1][y+1][z+1].p);
			gluggVertexv(lapp[1][y+1][z].p);
		}

	gluggColor(1, 1, 1);
	for (int y = 1; y <= kLappSizeY-1; y++)
		for (int z = 1; z <= kLappSizeZ-1; z++)
		{
			gluggNormalv(cross(lapp[kLappSizeX][y+1][z].p - lapp[kLappSizeX][y][z].p, lapp[kLappSizeX][y][z+1].p - lapp[kLappSizeX][y][z].p));
			gluggVertexv(lapp[kLappSizeX][y+1][z].p);
			gluggVertexv(lapp[kLappSizeX][y+1][z+1].p);
			gluggVertexv(lapp[kLappSizeX][y][z+1].p);
			gluggVertexv(lapp[kLappSizeX][y][z].p);
		}
	
	// Y sides
	gluggColor(1, 0.5, 0);
	for (int x = 1; x <= kLappSizeX-1; x++)
		for (int z = 1; z <= kLappSizeZ-1; z++)
		{
			gluggNormalv(cross(lapp[x+1][1][z].p - lapp[x][1][z].p, lapp[x][1][z+1].p - lapp[x][1][z].p));
			gluggVertexv(lapp[x][1][z].p);
			gluggVertexv(lapp[x][1][z+1].p);
			gluggVertexv(lapp[x+1][1][z+1].p);
			gluggVertexv(lapp[x+1][1][z].p);
		}

	gluggColor(0, 1, 1);
	for (int x = 1; x <= kLappSizeX-1; x++)
		for (int z = 1; z <= kLappSizeZ-1; z++)
		{
			gluggNormalv(cross(lapp[x][kLappSizeY][z+1].p - lapp[x][kLappSizeY][z].p, lapp[x+1][kLappSizeY][z].p - lapp[x][kLappSizeY][z].p));
			gluggVertexv(lapp[x+1][kLappSizeY][z].p);
			gluggVertexv(lapp[x+1][kLappSizeY][z+1].p);
			gluggVertexv(lapp[x][kLappSizeY][z+1].p);
			gluggVertexv(lapp[x][kLappSizeY][z].p);
		}

	shell = gluggBuildModel(1);
}

void Animate()
{
	if (animation)
	{
	
// Zero forces, add gravity.
	for (int x = 1; x <= kLappSizeX; x++)
		for (int y = 1; y <= kLappSizeY; y++)
			for (int z = 1; z <= kLappSizeZ; z++)
			{
				lapp[x][y][z].f.x = 0;
				lapp[x][y][z].f.y = -0.03; //gravitation
				lapp[x][y][z].f.z = 0;
			};

// Create forces from positions
	for (int x = 1; x <= kLappSizeX; x++)
		for (int y = 1; y <= kLappSizeY; y++)
			for (int z = 1; z <= kLappSizeZ; z++)		
				for (int dx = -1; dx <= 1; dx++)
					for (int dy = -1; dy <= 1; dy++)
						for (int dz = -1; dz <= 1; dz++)
							if (dx != 0 || dy != 0 || dz != 0)
			{
				if (spring[x][y][z][dx+1][dy+1][dz+1].nod1 != NULL)
					CalcForce(spring[x][y][z][dx+1][dy+1][dz+1]);
			};

// for all nodes in "lapp"
	for (int x = 1; x <= kLappSizeX; x++)
		for (int y = 1; y <= kLappSizeY; y++)
		for (int z = 1; z <= kLappSizeZ; z++)		
		{
			nextold = lapp[x][y][z].p;

			if (euler)
//*** Model 1: Euler integration
			{
// Calculate acceleration,
				lapp[x][y][z].a = lapp[x][y][z].f / lapp[x][y][z].m;

// Add to velocity
				lapp[x][y][z].v = lapp[x][y][z].v + lapp[x][y][z].a;

// Add velocity to position
				lapp[x][y][z].p = lapp[x][y][z].p + lapp[x][y][z].v;
			}
			else
			{
//*** Model 2: Verlet integration

// Calculate acceleration,
				lapp[x][y][z].a = lapp[x][y][z].f / lapp[x][y][z].m;
				
				vv = lapp[x][y][z].p - lapp[x][y][z].oldp;
				lapp[x][y][z].p = lapp[x][y][z].p + vv + lapp[x][y][z].a;

// This step needs lapp[x][y][z].oldp
			};

// Limit position to the frame
			if (lapp[x][y][z].p.x < ram.left)
				{
					lapp[x][y][z].p.x = ram.left;
					if (kBounce)
						lapp[x][y][z].v.x = fabs(lapp[x][y][z].v.x);
					else if (kPlast)
						lapp[x][y][z].v.x = 0;
				};
			if (lapp[x][y][z].p.x > ram.right)
				{
					lapp[x][y][z].p.x = ram.right;
					if (kBounce)
						lapp[x][y][z].v.x = -fabs(lapp[x][y][z].v.x);
					else if (kPlast)
						lapp[x][y][z].v.x = 0;
				};
			if (lapp[x][y][z].p.y < ram.top)
				{
					lapp[x][y][z].p.y = ram.top;
					if (kBounce)
						lapp[x][y][z].v.y = fabs(lapp[x][y][z].v.y);
					else if (kPlast)
						lapp[x][y][z].v.y = 0;
				};
			if (lapp[x][y][z].p.y > ram.bottom)
				{
					lapp[x][y][z].p.y = ram.bottom;
					if (kBounce)
						lapp[x][y][z].v.y = -fabs(lapp[x][y][z].v.y);
					else if (kPlast)
						lapp[x][y][z].v.y = 0;
				};
			if (lapp[x][y][z].p.z < ram.near)
				{
					lapp[x][y][z].p.z = ram.near;
					if (kBounce)
						lapp[x][y][z].v.z = fabs(lapp[x][y][z].v.z);
					else if (kPlast)
						lapp[x][y][z].v.z = 0;
				};
			if (lapp[x][y][z].p.z > ram.far)
				{
					lapp[x][y][z].p.z = ram.far;
					if (kBounce)
						lapp[x][y][z].v.z = -fabs(lapp[x][y][z].v.z);
					else if (kPlast)
						lapp[x][y][z].v.z = 0;
				};
			lapp[x][y][z].oldp = nextold;
		};
	gluggDisposeModel(shell);
	BuildShell();
	}; // if (animation)
}

#define ballRadius 5

void drawModelAt(gluggModel mdl, GLuint shader, mat4 mtx)
{
	glUseProgram(shader);
    glUniformMatrix4fv(glGetUniformLocation(shader, "modelviewMatrix"), 1, GL_TRUE, mtx.m);
	gluggDrawModel(mdl, shader);
}

int drawParticles = 1;
int drawShell = 0;

void DrawSpringSystem(mat4 cam)
{
	// Clear stencil
	glClearStencil(0); // Specify value for glClear to set stencil to.
	glClear(GL_STENCIL_BUFFER_BIT);
	glStencilOp(GL_KEEP, GL_KEEP, GL_INCR);
	glStencilFunc(GL_EQUAL, 0, 0xFFFFFFFF);

// Draw particles:
	if (drawParticles)
	{
		for (int x = 1; x <= kLappSizeX; x++)
			for (int y = 1; y <= kLappSizeY; y++)
			for (int z = 1; z <= kLappSizeZ; z++)		
				{
					// Draw small beachball at lapp[x][y][z].p
					drawModelAt(badboll, program, cam * T(lapp[x][y][z].p.x, lapp[x][y][z].p.y, lapp[x][y][z].p.z) * S(ballRadius, ballRadius, ballRadius));
				};

		// Draw shadows
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glEnable(GL_STENCIL_TEST);
		
		for (int x = 1; x <= kLappSizeX; x++)
			for (int y = 1; y <= kLappSizeY; y++)
			for (int z = 1; z <= kLappSizeZ; z++)		
		{
			drawModelAt(badboll, shadowprogram, cam * T(lapp[x][y][z].p.x, 1.1-ballRadius, lapp[x][y][z].p.z) * S(ballRadius, 0, ballRadius));
		}
		glDisable(GL_BLEND);
		glDisable(GL_STENCIL_TEST);
	}
	
	if (drawShell)
	{
		drawModelAt(shell, program, cam);

		glEnable(GL_STENCIL_TEST);

		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		drawModelAt(shell, shadowprogram, cam * S(1, 0.01, 1));
		glDisable(GL_BLEND);
		glDisable(GL_STENCIL_TEST);
	}
};

Rect MakeRect(float left, float top, float near, float right, float bottom, float far)
{
	Rect r;
	r.left = left;
	r.top = top;
	r.right = right;
	r.bottom = bottom;
	r.near = near;
	r.far = far;
	return r;
}

void initSystem()
{
// Initialize. Zero and place.
	for (int x = 1; x <= kLappSizeX; x++)
		for (int y = 1; y <= kLappSizeY; y++)
			for (int z = 1; z <= kLappSizeZ; z++)		
			{
				lapp[x][y][z].p.x = 50 + kNodeDist * x;
				lapp[x][y][z].p.y = 0 + kNodeDist * y + startHeight;
				lapp[x][y][z].p.z = 100 + kNodeDist * z;
				lapp[x][y][z].v.x = Rand(1.0) - 1;
				lapp[x][y][z].v.y = Rand(1.0) - 0.5;
				lapp[x][y][z].v.z = Rand(1.0) - 0.5;
				lapp[x][y][z].m = 1;
				lapp[x][y][z].oldp = lapp[x][y][z].p - lapp[x][y][z].v;
			};
// Create springs
	for (int x = 1; x <= kLappSizeX; x++)
		for (int y = 1; y <= kLappSizeY; y++)
			for (int z = 1; z <= kLappSizeZ; z++)		
				for (int dx = -1; dx <= 1; dx++)
					for (int dy = -1; dy <= 1; dy++)
						for (int dz = -1; dz <= 1; dz++)
						{
							if (dx != 0 || dy != 0 || dz != 0)
							{
								spring[x][y][z][dx+1][dy+1][dz+1] = SkapaFjader(x, y, z, x + dx, y + dy, z + dz);
							}
						}
}




float g = 9.81;

void display(void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    mat4 cam = lookAt(150, 100, 450,   150, 100, 0,   0,1,0);
	
	Animate();
	DrawSpringSystem(cam);

	drawModelAt(floorModel, program, cam);
	
	sgDraw();

    glutSwapBuffers();
}

void Start()
{
	animation = 1;
}

void Reset()
{
	initSystem();
	animation = 0;
	gluggDisposeModel(shell);
	BuildShell();
}

float kick = 10.0;

void Kick()
{
	int x = rand() % kLappSizeX + 1;
	int y = rand() % kLappSizeY + 1;
	int z = rand() % kLappSizeZ + 1;
	lapp[x][y][z].v.y += kick;
}

void myinit()
{
    glEnable(GL_DEPTH_TEST); /* enable z buffer */
    glDisable(GL_CULL_FACE);

	program = loadShaders("sphere.vert", "sphere.frag");
	shadowprogram = loadShaders("sphere.vert", "shadow.frag");

	badboll = badbollSphere(6, 10, 20, 1.0);

	ram = MakeRect(1, 1, -50, 300, 300, 200);
	Rect ramplus = MakeRect(1-ballRadius, 1-ballRadius, 1-ballRadius, 300+ballRadius, 300+ballRadius, 300+ballRadius);

	// Make floor
	gluggBegin(GLUGG_QUADS);
	gluggNormal(0,1,0);
	gluggColor(0.45, 0.95, 0.44);
	gluggColor(0.4, 1.0, 0.3);
	gluggVertex(ramplus.right, ramplus.top, 200); // The top is the bottom... for historical reasons
	gluggVertex(ramplus.right, ramplus.top, -50);
	gluggVertex(ramplus.left, ramplus.top, -50);
	gluggVertex(ramplus.left, ramplus.top, 200);
	// Left wall
	gluggNormal(1,0,0);
//	gluggColor(0.55, 1, 0.65);
//	gluggColor(0.3, 0.8, 0.4);
	gluggVertex(ramplus.left, ramplus.top, 200);
	gluggVertex(ramplus.left, ramplus.top, -50);
	gluggVertex(ramplus.left, ramplus.bottom, -50);
	gluggVertex(ramplus.left, ramplus.bottom, 200);
	// Right wall
	gluggNormal(-1,0,0); // ???
//	gluggColor(0.5, 1, 0.6);
//	gluggColor(0.3, 0.8, 0.4);
	gluggVertex(ramplus.right, ramplus.bottom, 200);
	gluggVertex(ramplus.right, ramplus.top, 200);
	gluggVertex(ramplus.right, ramplus.top, -50);
	gluggVertex(ramplus.right, ramplus.bottom, -50);
	// Far wall
	gluggNormal(0,0,1);
//	gluggColor(0.4, 0.9, 0.5);
	gluggVertex(ramplus.right, ramplus.top, -50);
	gluggVertex(ramplus.left, ramplus.top, -50);
	gluggVertex(ramplus.left, ramplus.bottom, -50);
	gluggVertex(ramplus.right, ramplus.bottom, -50);
	floorModel = gluggBuildModel(1);
	
	projectionMatrix = frustum(-0.5, 0.5, -0.5, 0.5, 1.0, 1000.0);
	
	glUseProgram(program);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	glUseProgram(shadowprogram);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);

    glClearColor (0.5, 0.8, 1.0, 1.0);
    
    sgSetPosition(10, 10);
    sgSetTextColor(0,0,0);
    sgSetBackgroundColor(0.5, 0.5, 0.5, 0.5);
    sgSetFrameColor(0,0,0);

    sgCreateStaticString(-1, -1, "SPROING");
    sgCreateDisplayFloat(-1, -1, "Starting height: ", &startHeight);
    sgCreateSlider(-1, -1, 200, &startHeight, 0, 200);
    sgCreateDisplayFloat(-1, -1, "Spring constant: ", &ks);
    sgCreateSlider(-1, -1, 200, &ks, 0, 0.5);
    sgCreateDisplayFloat(-1, -1, "Dampening factor: ", &kd);
    sgCreateSlider(-1, -1, 200, &kd, 0, 0.1);
    sgCreateDisplayFloat(-1, -1, "Kick strength: ", &kick);
    sgCreateSlider(-1, -1, 200, &kick, 10, 100);
   
// Options for skipping springs are not yet implemented
//	sgCreateRadio(-1, -1, "4 springs", &numSprings, 4);
//	sgCreateRadio(-1, -1, "6 springs", &numSprings, 6);
//	sgCreateRadio(-1, -1, "8 springs", &numSprings, 8);
	
    sgCreateCheckBox(-1, -1, "Euler", &euler);
    sgCreateCheckBox(-1, -1, "Draw particles", &drawParticles);
    sgCreateCheckBox(-1, -1, "Draw shell", &drawShell);
    sgCreateButton(-1, -1, "Start", Start);
	sgCreateButton(-1, -1, "Reset", Reset);
	sgCreateButton(-1, -1, "Kick", Kick);

	initSystem();
	BuildShell();

	Reset();
}

void mouse(int button, int state, int x, int y)
{
	sgMouse(state, x, y);
}

void mousedrag(int x, int y)
{
	sgMouseDrag(x, y);
}

void Key(unsigned char key, int x, int y)
{
	if (key == ' ')
		animation = !animation;
	if (key == 'e')
		euler = 1;
	if (key == 'v')
		euler = 0;
	if (key == 'r')
		Reset();
	if (key == 'p')
		drawParticles = !drawParticles;
	if (key == 's')
		drawShell = !drawShell;
};


int main(int argc, char **argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH | GLUT_STENCIL);
    glutInitContextVersion(3, 2);
    glutInitWindowSize(500, 500);
    glutRepeatingTimer(30);
    glutCreateWindow("Sproing 3D");
    myinit();
    glutDisplayFunc(display);
    glutMouseFunc(mouse);
    glutMotionFunc(mousedrag);
    glutKeyboardFunc(Key);
    glutMainLoop();
}

