// Wind
// Plan: Flat panels rotating in the wind

// uses framework Cocoa
// uses framework OpenGL
#define MAIN
#include <stdlib.h>
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils4.h"
#include "glugg.h"
#include "gluggShapes.h"
#include "SimpleGUI.h"


GLuint program, shadowprogram; // Shader
mat4 projectionMatrix;

// Modified gluggCube
void createBox(GLfloat sizex, GLfloat sizey, GLfloat sizez)
{
  static vec3 n[6] =
  {
    vec3(-1.0, 0.0, 0.0),
    vec3(0.0, 1.0, 0.0),
    vec3(1.0, 0.0, 0.0),
    vec3(0.0, -1.0, 0.0),
    vec3(0.0, 0.0, 1.0),
    vec3(0.0, 0.0, -1.0)
  };
  static GLint faces[6][4] =
  {
    {0, 1, 2, 3},
    {3, 2, 6, 7},
    {7, 6, 5, 4},
    {4, 5, 1, 0},
    {5, 6, 2, 1},
    {7, 4, 0, 3}
  };
  vec3 v[8] =
  {
  	vec3(-sizex/2,-sizey/2,-sizez/2),
  	vec3(-sizex/2,-sizey/2, sizez/2),
  	vec3(-sizex/2, sizey/2, sizez/2),
  	vec3(-sizex/2, sizey/2,-sizez/2),
  	vec3( sizex/2,-sizey/2,-sizez/2),
  	vec3( sizex/2,-sizey/2, sizez/2),
  	vec3( sizex/2, sizey/2, sizez/2),
  	vec3( sizex/2, sizey/2,-sizez/2)
  };
  GLint i;
  
  gluggMode(GLUGG_QUADS);

  for (i = 5; i >= 0; i--)
  {
    gluggNormalv(n[i]);
    gluggVertexv(v[faces[i][0]]);
    gluggVertexv(v[faces[i][1]]);
    gluggVertexv(v[faces[i][2]]);
    gluggVertexv(v[faces[i][3]]);
  }
}

gluggModel plank, floorModel;

vec3 plankPosition;
vec3 plankVelocity;
float rot, omega;

float g = 9.81;
int check = 0;
int check2 = 0;
float plankMass = 2.0;
float l = 0.0;
float startVelocity = 0.1;
float startRotation = 1.0;
float startOmega = 0.03;
//float startPosition = 0.0;
vec3 windDirection = vec3(1, 0, 0);

void drawModelAt(gluggModel mdl, GLuint shader, mat4 mtx)
{
	glUseProgram(shader);
	glUniformMatrix4fv(glGetUniformLocation(shader, "modelviewMatrix"), 1, GL_TRUE, mtx.m);
	gluggDrawModel(mdl, shader);
}

void display(void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    mat4 cam = lookAt(0, 7, 14, 0, 4, 0, 0,1,0);
	
	// Update velocity from rotation and wind
	vec3 up = vec3(cos(rot), -sin(rot), 0);
	if (up.x < 0) up = -up;
	printf("rot %f, up = %f %f\n", rot, up.x, up.y);
	
	float a = fabs(up.y) * 5*4; // Area seen from wind direction. We know that the plank is 5*4
	
	vec3 windUp, windTangent;
	SplitVector(windDirection, up, &windUp, &windTangent);
	windUp.y *= 5.0;
	plankVelocity += windUp * a * 0.0001;
	printVec3(windUp);
	
	plankPosition = plankPosition + plankVelocity; // *dt missing
	rot = rot + omega;
	
	mat4 rotm = Rz(rot);
	drawModelAt(plank, program, cam * T(plankPosition.x, plankPosition.y, plankPosition.z) * rotm);
	drawModelAt(floorModel, program, cam);

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	drawModelAt(plank, shadowprogram, cam * T(plankPosition.x,0.1,plankPosition.z) * S(1,0,1) * rotm);
	glDisable(GL_BLEND);
	
	sgDraw();

    glutSwapBuffers();
}

void Reset()
{
	// plank
	plankPosition = vec3(-3,4,0);
	plankVelocity = vec3(0,0,0);
	rot = startRotation;
	omega = startOmega; // rotation speed
	l = 0.0; // rotational momentum
	// axis
//	axisPos = vec3(axisPosition, 0, 0);
}

void Start()
{
}

void myinit()
{
    glEnable(GL_DEPTH_TEST); /* enable z buffer */
    glEnable(GL_CULL_FACE);

	program = loadShaders("sphere.vert", "sphere.frag");
	shadowprogram = loadShaders("sphere.vert", "shadow.frag");

	gluggBegin(GLUGG_QUADS);
		gluggColor(1.0, 1.0, 0.1);
		createBox(5, 0.2, 4);
	plank = gluggBuildModel(1);

	// Make floor
	gluggBegin(GLUGG_QUADS);
	gluggNormal(0,1,0);
	gluggColor(0.5, 1, 0.6);
	gluggVertex(25, 0, 5);
	gluggVertex(25, 0, -15);
	gluggVertex(-25, 0, -15);
	gluggVertex(-25, 0, 5);
	floorModel = gluggBuildModel(1);

	projectionMatrix = frustum(-0.5, 0.5, -0.5, 0.5, 1.0, 60.0);
	
	glUseProgram(program);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	glUseProgram(shadowprogram);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);

    glClearColor (1.0, 1.0, 1.0, 1.0);
    
    sgSetPosition(30, 30);
    sgSetTextColor(0,0,0);
    sgSetBackgroundColor(0.5, 0.5, 0.5, 0.5);
    sgSetFrameColor(0,0,0);
    sgCreateStaticString(20, 20, "WIND");
//    sgCreateDisplayFloat(-1, -1, "Starting velocity: ", &startVelocity);
//	sgCreateSlider(-1, -1, 200, &startVelocity, 0.1, 1.5);
    sgCreateDisplayFloat(-1, -1, "Starting rotation: ", &startRotation);
	sgCreateSlider(-1, -1, 200, &startRotation, 0.1, 3);
    sgCreateDisplayFloat(-1, -1, "Starting rotation speed: ", &startOmega);
	sgCreateSlider(-1, -1, 200, &startOmega, 0.01, 0.2);
    sgCreateButton(-1, -1, "Reset", Reset);
    sgCreateButton(-1, -1, "Start", Start);

	Reset();
}


void mouse(int button, int state, int x, int y)
{
	sgMouse(state, x, y);
}

void mousedrag(int x, int y)
{
	sgMouseDrag(x, y);
}


int main(int argc, char **argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitContextVersion(3, 2);
    glutInitWindowSize(500, 500);
    glutRepeatingTimer(15);
    glutCreateWindow("Wind");
    myinit();
    glutDisplayFunc(display);
    glutMouseFunc(mouse);
    glutMotionFunc(mousedrag);
    glutMainLoop();
}
